@@ -9,7 +9,6 @@
 #include "adw-carousel.h"
 
 #include "adw-animation-util.h"
-#include "adw-macros-private.h"
 #include "adw-navigation-direction.h"
 #include "adw-spring-animation.h"
 #include "adw-swipe-tracker.h"
@@ -40,8 +39,6 @@
  * ## CSS nodes
  *
  * `AdwCarousel` has a single CSS node with name `carousel`.
- *
- * Since: 1.0
  */
 
 typedef struct {
@@ -328,8 +325,15 @@ animate_child_resize (AdwCarousel *self,
 
   update_shift_position_flag (self, child);
 
-  if (child->resize_animation)
+  if (child->resize_animation) {
+    gboolean been_removing = child->removing;
     adw_animation_skip (child->resize_animation);
+    /* It's because the skip finishes the animation, which triggers
+       the 'done' signal, which calls resize_animation_done_cb(),
+       which frees the 'child' immediately. */
+    if (been_removing)
+      return;
+  }
 
   target = adw_callback_animation_target_new ((AdwAnimationTargetFunc)
                                               resize_animation_value_cb,
@@ -590,7 +594,7 @@ adw_carousel_size_allocate (GtkWidget *widget,
   gboolean is_rtl;
   double snap_point;
 
-  if (self->position_shift != 0) {
+  if (!G_APPROX_VALUE (self->position_shift, 0, DBL_EPSILON)) {
     set_position (self, self->position + self->position_shift);
     adw_swipe_tracker_shift_position (self->tracker, self->position_shift);
     self->position_shift = 0;
@@ -880,8 +884,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * AdwCarousel:n-pages: (attributes org.gtk.Property.get=adw_carousel_get_n_pages)
    *
    * The number of pages in a `AdwCarousel`.
-   *
-   * Since: 1.0
    */
   props[PROP_N_PAGES] =
     g_param_spec_uint ("n-pages", NULL, NULL,
@@ -896,8 +898,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * Current scrolling position, unitless.
    *
    * 1 matches 1 page. Use [method@Carousel.scroll_to] for changing it.
-   *
-   * Since: 1.0
    */
   props[PROP_POSITION] =
     g_param_spec_double ("position", NULL, NULL,
@@ -911,10 +911,8 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    *
    * Whether the carousel can be navigated.
    *
-   * This can be used to temporarily disable a `AdwCarousel` to only allow
+   * This can be used to temporarily disable the carousel to only allow
    * navigating it in a certain state.
-   *
-   * Since: 1.0
    */
   props[PROP_INTERACTIVE] =
     g_param_spec_boolean ("interactive", NULL, NULL,
@@ -925,8 +923,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * AdwCarousel:spacing: (attributes org.gtk.Property.get=adw_carousel_get_spacing org.gtk.Property.set=adw_carousel_set_spacing)
    *
    * Spacing between pages in pixels.
-   *
-   * Since: 1.0
    */
   props[PROP_SPACING] =
     g_param_spec_uint ("spacing", NULL, NULL,
@@ -945,8 +941,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * ```c
    * adw_spring_params_new (1, 0.5, 500)
    * ```
-   *
-   * Since: 1.0
    */
   props[PROP_SCROLL_PARAMS] =
     g_param_spec_boxed ("scroll-params", NULL, NULL,
@@ -959,8 +953,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * Sets whether the `AdwCarousel` can be dragged with mouse pointer.
    *
    * If the value is `FALSE`, dragging is only available on touch.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_MOUSE_DRAG] =
     g_param_spec_boolean ("allow-mouse-drag", NULL, NULL,
@@ -973,8 +965,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * Whether the widget will respond to scroll wheel events.
    *
    * If the value is `FALSE`, wheel events will be ignored.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_SCROLL_WHEEL] =
     g_param_spec_boolean ("allow-scroll-wheel", NULL, NULL,
@@ -987,8 +977,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    * Whether to allow swiping for more than one page at a time.
    *
    * If the value is `FALSE`, each swipe can only move to the adjacent pages.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_LONG_SWIPES] =
     g_param_spec_boolean ("allow-long-swipes", NULL, NULL,
@@ -1000,7 +988,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    *
    * Page reveal duration, in milliseconds.
    *
-   * Since: 1.0
+   * Reveal duration is used when animating adding or removing pages.
    */
   props[PROP_REVEAL_DURATION] =
     g_param_spec_uint ("reveal-duration", NULL, NULL,
@@ -1024,8 +1012,6 @@ adw_carousel_class_init (AdwCarouselClass *klass)
    *
    * It can be used to implement "infinite scrolling" by amending the pages
    * after every scroll.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_PAGE_CHANGED] =
     g_signal_new ("page-changed",
@@ -1162,8 +1148,6 @@ adw_carousel_swipeable_init (AdwSwipeableInterface *iface)
  * Creates a new `AdwCarousel`.
  *
  * Returns: the newly created `AdwCarousel`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_carousel_new (void)
@@ -1177,8 +1161,6 @@ adw_carousel_new (void)
  * @child: a widget to add
  *
  * Prepends @child to @self.
- *
- * Since: 1.0
  */
 void
 adw_carousel_prepend (AdwCarousel *self,
@@ -1196,8 +1178,6 @@ adw_carousel_prepend (AdwCarousel *self,
  * @child: a widget to add
  *
  * Appends @child to @self.
- *
- * Since: 1.0
  */
 void
 adw_carousel_append (AdwCarousel *self,
@@ -1219,8 +1199,6 @@ adw_carousel_append (AdwCarousel *self,
  *
  * If position is -1, or larger than the number of pages,
  * @child will be appended to the end.
- *
- * Since: 1.0
  */
 void
 adw_carousel_insert (AdwCarousel *self,
@@ -1268,8 +1246,6 @@ adw_carousel_insert (AdwCarousel *self,
  *
  * If position is -1, or larger than the number of pages, @child will be moved
  * at the end.
- *
- * Since: 1.0
  */
 void
 adw_carousel_reorder (AdwCarousel *self,
@@ -1340,11 +1316,13 @@ adw_carousel_reorder (AdwCarousel *self,
     gtk_widget_insert_before (child, GTK_WIDGET (self), NULL);
   }
 
-  if (closest_point == old_point)
+  if (G_APPROX_VALUE (closest_point, old_point, DBL_EPSILON))
     self->position_shift += new_point - old_point;
-  else if (old_point >= closest_point && closest_point >= new_point)
+  else if ((G_APPROX_VALUE (old_point, closest_point, DBL_EPSILON) || old_point > closest_point) &&
+           (G_APPROX_VALUE (closest_point, new_point, DBL_EPSILON) || closest_point > new_point))
     self->position_shift += info->size;
-  else if (new_point >= closest_point && closest_point >= old_point)
+  else if ((G_APPROX_VALUE (new_point, closest_point, DBL_EPSILON) || new_point > closest_point) &&
+           (G_APPROX_VALUE (closest_point, old_point, DBL_EPSILON) || closest_point > old_point))
     self->position_shift -= info->size;
 
   gtk_widget_queue_allocate (GTK_WIDGET (self));
@@ -1356,8 +1334,6 @@ adw_carousel_reorder (AdwCarousel *self,
  * @child: a widget to remove
  *
  * Removes @child from @self.
- *
- * Since: 1.0
  */
 void
 adw_carousel_remove (AdwCarousel *self,
@@ -1394,8 +1370,6 @@ adw_carousel_remove (AdwCarousel *self,
  * Scrolls to @widget.
  *
  * If @animate is `TRUE`, the transition will be animated.
- *
- * Since: 1.0
  */
 void
 adw_carousel_scroll_to (AdwCarousel *self,
@@ -1420,8 +1394,6 @@ adw_carousel_scroll_to (AdwCarousel *self,
  * Gets the page at position @n.
  *
  * Returns: (transfer none): the page
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_carousel_get_nth_page (AdwCarousel *self,
@@ -1444,8 +1416,6 @@ adw_carousel_get_nth_page (AdwCarousel *self,
  * Gets the number of pages in @self.
  *
  * Returns: the number of pages in @self
- *
- * Since: 1.0
  */
 guint
 adw_carousel_get_n_pages (AdwCarousel *self)
@@ -1470,13 +1440,11 @@ adw_carousel_get_n_pages (AdwCarousel *self)
  * adw_carousel_get_position: (attributes org.gtk.Method.get_property=position)
  * @self: a carousel
  *
- * Gets current scroll position in @self.
+ * Gets current scroll position in @self, unitless.
  *
- * It's unitless, 1 matches 1 page.
+ * 1 matches 1 page. Use [method@Carousel.scroll_to] for changing it.
  *
  * Returns: the scroll position
- *
- * Since: 1.0
  */
 double
 adw_carousel_get_position (AdwCarousel *self)
@@ -1493,8 +1461,6 @@ adw_carousel_get_position (AdwCarousel *self)
  * Gets whether @self can be navigated.
  *
  * Returns: whether @self can be navigated
- *
- * Since: 1.0
  */
 gboolean
 adw_carousel_get_interactive (AdwCarousel *self)
@@ -1511,7 +1477,8 @@ adw_carousel_get_interactive (AdwCarousel *self)
  *
  * Sets whether @self can be navigated.
  *
- * Since: 1.0
+ * This can be used to temporarily disable the carousel to only allow navigating
+ * it in a certain state.
  */
 void
 adw_carousel_set_interactive (AdwCarousel *self,
@@ -1536,8 +1503,6 @@ adw_carousel_set_interactive (AdwCarousel *self,
  * Gets spacing between pages in pixels.
  *
  * Returns: spacing between pages
- *
- * Since: 1.0
  */
 guint
 adw_carousel_get_spacing (AdwCarousel *self)
@@ -1553,8 +1518,6 @@ adw_carousel_get_spacing (AdwCarousel *self)
  * @spacing: the new spacing value
  *
  * Sets spacing between pages in pixels.
- *
- * Since: 1.0
  */
 void
 adw_carousel_set_spacing (AdwCarousel *self,
@@ -1578,8 +1541,6 @@ adw_carousel_set_spacing (AdwCarousel *self,
  * Gets the scroll animation spring parameters for @self.
  *
  * Returns: the animation parameters
- *
- * Since: 1.0
  */
 AdwSpringParams *
 adw_carousel_get_scroll_params (AdwCarousel *self)
@@ -1596,7 +1557,11 @@ adw_carousel_get_scroll_params (AdwCarousel *self)
  *
  * Sets the scroll animation spring parameters for @self.
  *
- * Since: 1.0
+ * The default value is equivalent to:
+ *
+ * ```c
+ * adw_spring_params_new (1, 0.5, 500)
+ * ```
  */
 void
 adw_carousel_set_scroll_params (AdwCarousel     *self,
@@ -1620,8 +1585,6 @@ adw_carousel_set_scroll_params (AdwCarousel     *self,
  * Sets whether @self can be dragged with mouse pointer.
  *
  * Returns: whether @self can be dragged with mouse pointer
- *
- * Since: 1.0
  */
 gboolean
 adw_carousel_get_allow_mouse_drag (AdwCarousel *self)
@@ -1638,7 +1601,7 @@ adw_carousel_get_allow_mouse_drag (AdwCarousel *self)
  *
  * Sets whether @self can be dragged with mouse pointer.
  *
- * Since: 1.0
+ * If @allow_mouse_drag is `FALSE`, dragging is only available on touch.
  */
 void
 adw_carousel_set_allow_mouse_drag (AdwCarousel *self,
@@ -1663,8 +1626,6 @@ adw_carousel_set_allow_mouse_drag (AdwCarousel *self,
  * Gets whether @self will respond to scroll wheel events.
  *
  * Returns: `TRUE` if @self will respond to scroll wheel events
- *
- * Since: 1.0
  */
 gboolean
 adw_carousel_get_allow_scroll_wheel (AdwCarousel *self)
@@ -1681,7 +1642,7 @@ adw_carousel_get_allow_scroll_wheel (AdwCarousel *self)
  *
  * Sets whether @self will respond to scroll wheel events.
  *
- * Since: 1.0
+ * If @allow_scroll_wheel is `FALSE`, wheel events will be ignored.
  */
 void
 adw_carousel_set_allow_scroll_wheel (AdwCarousel *self,
@@ -1706,8 +1667,6 @@ adw_carousel_set_allow_scroll_wheel (AdwCarousel *self,
  * Gets whether to allow swiping for more than one page at a time.
  *
  * Returns: `TRUE` if long swipes are allowed
- *
- * Since: 1.0
  */
 gboolean
 adw_carousel_get_allow_long_swipes (AdwCarousel *self)
@@ -1724,7 +1683,8 @@ adw_carousel_get_allow_long_swipes (AdwCarousel *self)
  *
  * Sets whether to allow swiping for more than one page at a time.
  *
- * Since: 1.0
+ * If @allow_long_swipes is `FALSE`, each swipe can only move to the adjacent
+ * pages.
  */
 void
 adw_carousel_set_allow_long_swipes (AdwCarousel *self,
@@ -1746,11 +1706,9 @@ adw_carousel_set_allow_long_swipes (AdwCarousel *self,
  * adw_carousel_get_reveal_duration: (attributes org.gtk.Method.get_property=reveal-duration)
  * @self: a carousel
  *
- * Gets duration of the animation used when adding or removing pages.
+ * Gets the page reveal duration, in milliseconds.
  *
  * Returns: the duration
- *
- * Since: 1.0
  */
 guint
 adw_carousel_get_reveal_duration (AdwCarousel *self)
@@ -1765,9 +1723,9 @@ adw_carousel_get_reveal_duration (AdwCarousel *self)
  * @self: a carousel
  * @reveal_duration: the new reveal duration value
  *
- * Sets duration of the animation used when adding or removing pages.
+ * Sets the page reveal duration, in milliseconds.
  *
- * Since: 1.0
+ * Reveal duration is used when animating adding or removing pages.
  */
 void
 adw_carousel_set_reveal_duration (AdwCarousel *self,
