@@ -83,12 +83,14 @@ typedef struct
   gboolean activates_default;
 } AdwEntryRowPrivate;
 
-static void adw_entry_row_editable_init (GtkEditableInterface *iface);
 static void adw_entry_row_buildable_init (GtkBuildableIface *iface);
+static void adw_entry_row_accessible_init (GtkAccessibleInterface *iface);
+static void adw_entry_row_editable_init (GtkEditableInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (AdwEntryRow, adw_entry_row, ADW_TYPE_PREFERENCES_ROW,
                          G_ADD_PRIVATE (AdwEntryRow)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, adw_entry_row_buildable_init)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE, adw_entry_row_accessible_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_EDITABLE, adw_entry_row_editable_init))
 
 static GtkBuildableIface *parent_buildable_iface;
@@ -522,8 +524,7 @@ adw_entry_row_class_init (AdwEntryRowClass *klass)
    *
    * Whether to suggest emoji replacements on the entry row.
    *
-   * Emoji replacement is done with :-delimited names,
-   * like `:heart:`.
+   * Emoji replacement is done with :-delimited names, like `:heart:`.
    *
    * Since: 1.2
    */
@@ -663,6 +664,19 @@ adw_entry_row_buildable_init (GtkBuildableIface *iface)
   iface->add_child = adw_entry_row_buildable_add_child;
 }
 
+static gboolean
+adw_entry_row_accessible_get_platform_state (GtkAccessible              *accessible,
+                                             GtkAccessiblePlatformState  state)
+{
+  return gtk_editable_delegate_get_accessible_platform_state (GTK_EDITABLE (accessible), state);
+}
+
+static void
+adw_entry_row_accessible_init (GtkAccessibleInterface *iface)
+{
+  iface->get_platform_state = adw_entry_row_accessible_get_platform_state;
+}
+
 static GtkEditable *
 adw_entry_row_get_delegate (GtkEditable *editable)
 {
@@ -714,7 +728,7 @@ adw_entry_row_add_prefix (AdwEntryRow *self,
   priv = adw_entry_row_get_instance_private (self);
 
   gtk_box_prepend (priv->prefixes, widget);
-  gtk_widget_show (GTK_WIDGET (priv->prefixes));
+  gtk_widget_set_visible (GTK_WIDGET (priv->prefixes), TRUE);
 }
 
 /**
@@ -738,7 +752,7 @@ adw_entry_row_add_suffix (AdwEntryRow *self,
   priv = adw_entry_row_get_instance_private (self);
 
   gtk_box_append (priv->suffixes, widget);
-  gtk_widget_show (GTK_WIDGET (priv->suffixes));
+  gtk_widget_set_visible (GTK_WIDGET (priv->suffixes), TRUE);
 }
 
 /**
@@ -764,12 +778,13 @@ adw_entry_row_remove (AdwEntryRow *self,
 
   parent = gtk_widget_get_parent (child);
 
-  if (parent == GTK_WIDGET (priv->prefixes))
-    gtk_box_remove (priv->prefixes, child);
-  else if (parent == GTK_WIDGET (priv->suffixes))
-    gtk_box_remove (priv->suffixes, child);
-  else
+  if (parent == GTK_WIDGET (priv->prefixes) || parent == GTK_WIDGET (priv->suffixes)) {
+    gtk_box_remove (GTK_BOX (parent), child);
+    gtk_widget_set_visible (parent, gtk_widget_get_first_child (parent) != NULL);
+  }
+  else {
     ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+  }
 }
 
 /**
@@ -801,6 +816,14 @@ adw_entry_row_get_show_apply_button (AdwEntryRow *self)
  *
  * Sets whether @self can show the apply button.
  *
+ * When set to `TRUE`, typing text in the entry will reveal an apply button.
+ * Clicking it or pressing the <kbd>Enter</kbd> key will hide the button and
+ * emit the [signal@EntryRow::apply] signal.
+ *
+ * This is useful if changing the entry contents can trigger an expensive
+ * operation, e.g. network activity, to avoid triggering it after typing every
+ * character.
+ *
  * Since: 1.2
  */
 void
@@ -990,9 +1013,6 @@ adw_entry_row_set_attributes (AdwEntryRow   *self,
  *
  * Gets whether to suggest emoji replacements on @self.
  *
- * Emoji replacement is done with :-delimited names,
- * like `:heart:`.
- *
  * Returns: whether or not emoji completion is enabled
  *
  * Since: 1.2
@@ -1016,8 +1036,7 @@ adw_entry_row_get_enable_emoji_completion (AdwEntryRow *self)
  *
  * Sets whether to suggest emoji replacements on @self.
  *
- * Emoji replacement is done with :-delimited names,
- * like `:heart:`.
+ * Emoji replacement is done with :-delimited names, like `:heart:`.
  *
  * Since: 1.2
  */
@@ -1048,6 +1067,8 @@ adw_entry_row_set_enable_emoji_completion (AdwEntryRow *self,
  * Gets whether activating the embedded entry can activate the default widget.
  *
  * Returns: whether to activate the default widget
+ *
+ * Since: 1.2
  */
 gboolean
 adw_entry_row_get_activates_default (AdwEntryRow *self)
@@ -1130,3 +1151,27 @@ adw_entry_row_set_show_indicator (AdwEntryRow *self,
 
   update_empty (self);
 }
+
+/**
+ * adw_entry_row_grab_focus_without_selecting:
+ * @self: an entry row
+ *
+ * Causes @self to have keyboard focus without selecting the text.
+ * 
+ * See [method@Gtk.Text.grab_focus_without_selecting] for more information.
+ *
+ * Returns: whether the focus is now inside @self
+ *
+ * Since: 1.3
+ */
+gboolean
+adw_entry_row_grab_focus_without_selecting (AdwEntryRow *self)
+{
+  AdwEntryRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_ENTRY_ROW (self), FALSE);
+
+  priv = adw_entry_row_get_instance_private (self);
+
+  return gtk_text_grab_focus_without_selecting (GTK_TEXT (priv->text));
+}
