@@ -42,8 +42,6 @@
  * It contains the subnodes `row.header` for its main embedded row,
  * `list.nested` for the list it can expand, and `image.expander-row-arrow` for
  * its arrow.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -77,6 +75,8 @@ enum {
   PROP_EXPANDED,
   PROP_ENABLE_EXPANSION,
   PROP_SHOW_ENABLE_SWITCH,
+  PROP_TITLE_LINES,
+  PROP_SUBTITLE_LINES,
   LAST_PROP,
 };
 
@@ -130,6 +130,12 @@ adw_expander_row_get_property (GObject    *object,
   case PROP_SHOW_ENABLE_SWITCH:
     g_value_set_boolean (value, adw_expander_row_get_show_enable_switch (self));
     break;
+  case PROP_TITLE_LINES:
+    g_value_set_int (value, adw_expander_row_get_title_lines (self));
+    break;
+  case PROP_SUBTITLE_LINES:
+    g_value_set_int (value, adw_expander_row_get_subtitle_lines (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -159,6 +165,12 @@ adw_expander_row_set_property (GObject      *object,
   case PROP_SHOW_ENABLE_SWITCH:
     adw_expander_row_set_show_enable_switch (self, g_value_get_boolean (value));
     break;
+  case PROP_TITLE_LINES:
+    adw_expander_row_set_title_lines (self, g_value_get_int (value));
+    break;
+  case PROP_SUBTITLE_LINES:
+    adw_expander_row_set_subtitle_lines (self, g_value_get_int (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -181,7 +193,8 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    *
    * The subtitle for this row.
    *
-   * Since: 1.0
+   * The subtitle is interpreted as Pango markup unless
+   * [property@PreferencesRow:use-markup] is set to `FALSE`.
    */
   props[PROP_SUBTITLE] =
     g_param_spec_string ("subtitle", NULL, NULL,
@@ -193,7 +206,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    *
    * The icon name for this row.
    *
-   * Since: 1.0
+   * Deprecated: 1.3: Use [method@ExpanderRow.add_prefix] to add an icon.
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -204,8 +217,6 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    * AdwExpanderRow:expanded: (attributes org.gtk.Property.get=adw_expander_row_get_expanded org.gtk.Property.set=adw_expander_row_set_expanded)
    *
    * Whether the row is expanded.
-   *
-   * Since: 1.0
    */
   props[PROP_EXPANDED] =
     g_param_spec_boolean ("expanded", NULL, NULL,
@@ -216,8 +227,6 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    * AdwExpanderRow:enable-expansion: (attributes org.gtk.Property.get=adw_expander_row_get_enable_expansion org.gtk.Property.set=adw_expander_row_set_enable_expansion)
    *
    * Whether expansion is enabled.
-   *
-   * Since: 1.0
    */
   props[PROP_ENABLE_EXPANSION] =
     g_param_spec_boolean ("enable-expansion", NULL, NULL,
@@ -228,14 +237,43 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    * AdwExpanderRow:show-enable-switch: (attributes org.gtk.Property.get=adw_expander_row_get_show_enable_switch org.gtk.Property.set=adw_expander_row_set_show_enable_switch)
    *
    * Whether the switch enabling the expansion is visible.
-   *
-   * Since: 1.0
    */
   props[PROP_SHOW_ENABLE_SWITCH] =
     g_param_spec_boolean ("show-enable-switch", NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwExpanderRow:title-lines: (attributes org.gtk.Property.get=adw_expander_row_get_title_lines org.gtk.Property.set=adw_expander_row_set_title_lines)
+   *
+   * The number of lines at the end of which the title label will be ellipsized.
+   *
+   * If the value is 0, the number of lines won't be limited.
+   *
+   * Since: 1.3
+   */
+  props[PROP_TITLE_LINES] =
+    g_param_spec_int ("title-lines", NULL, NULL,
+                      0, G_MAXINT,
+                      0,
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwExpanderRow:subtitle-lines: (attributes org.gtk.Property.get=adw_expander_row_get_subtitle_lines org.gtk.Property.set=adw_expander_row_set_subtitle_lines)
+   *
+   * The number of lines at the end of which the subtitle label will be
+   * ellipsized.
+   *
+   * If the value is 0, the number of lines won't be limited.
+   *
+   * Since: 1.3
+   */
+  props[PROP_SUBTITLE_LINES] =
+    g_param_spec_int ("subtitle-lines", NULL, NULL,
+                      0, G_MAXINT,
+                      0,
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -258,6 +296,8 @@ func (gpointer this) { \
 
 NOTIFY (notify_subtitle_cb, PROP_SUBTITLE);
 NOTIFY (notify_icon_name_cb, PROP_ICON_NAME);
+NOTIFY (notify_title_lines_cb, PROP_TITLE_LINES);
+NOTIFY (notify_subtitle_lines_cb, PROP_SUBTITLE_LINES);
 
 static void
 adw_expander_row_init (AdwExpanderRow *self)
@@ -273,6 +313,8 @@ adw_expander_row_init (AdwExpanderRow *self)
 
   g_signal_connect_object (priv->action_row, "notify::subtitle", G_CALLBACK (notify_subtitle_cb), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->action_row, "notify::icon-name", G_CALLBACK (notify_icon_name_cb), self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->action_row, "notify::title-lines", G_CALLBACK (notify_title_lines_cb), self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->action_row, "notify::subtitle-lines", G_CALLBACK (notify_subtitle_lines_cb), self, G_CONNECT_SWAPPED);
 }
 
 static void
@@ -310,8 +352,6 @@ adw_expander_row_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwExpanderRow`.
  *
  * Returns: the newly created `AdwExpanderRow`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_expander_row_new (void)
@@ -319,6 +359,118 @@ adw_expander_row_new (void)
   return g_object_new (ADW_TYPE_EXPANDER_ROW, NULL);
 }
 
+/**
+ * adw_expander_row_add_action:
+ * @self: an expander row
+ * @widget: a widget
+ *
+ * Adds an action widget to @self.
+ */
+void
+adw_expander_row_add_action (AdwExpanderRow *self,
+                             GtkWidget      *widget)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (self));
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  gtk_box_prepend (priv->actions, widget);
+  gtk_widget_set_visible (GTK_WIDGET (priv->actions), TRUE);
+}
+
+/**
+ * adw_expander_row_add_prefix:
+ * @self: an expander row
+ * @widget: a widget
+ *
+ * Adds a prefix widget to @self.
+ */
+void
+adw_expander_row_add_prefix (AdwExpanderRow *self,
+                             GtkWidget      *widget)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  if (priv->prefixes == NULL) {
+    priv->prefixes = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12));
+    adw_action_row_add_prefix (ADW_ACTION_ROW (priv->action_row), GTK_WIDGET (priv->prefixes));
+  }
+  gtk_box_append (priv->prefixes, widget);
+}
+
+/**
+ * adw_expander_row_add_row:
+ * @self: an expander row
+ * @child: a widget
+ *
+ * Adds a widget to @self.
+ *
+ * The widget will appear in the expanding list below @self.
+ */
+void
+adw_expander_row_add_row (AdwExpanderRow *self,
+                          GtkWidget      *child)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  /* When constructing the widget, we want the box to be added as the child of
+   * the GtkListBoxRow, as an implementation detail.
+   */
+  gtk_list_box_append (priv->list, child);
+
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "empty");
+}
+
+/**
+ * adw_action_row_expander:
+ * @self: an expander row
+ * @widget: the child to be removed
+ *
+ * Removes a child from @self.
+ */
+void
+adw_expander_row_remove (AdwExpanderRow *self,
+                         GtkWidget      *child)
+{
+  AdwExpanderRowPrivate *priv;
+  GtkWidget *parent;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  parent = gtk_widget_get_parent (child);
+
+  if (parent == GTK_WIDGET (priv->prefixes) || parent == GTK_WIDGET (priv->actions)) {
+    gtk_box_remove (GTK_BOX (parent), child);
+    gtk_widget_set_visible (parent, gtk_widget_get_first_child (parent) != NULL);
+  }
+  else if (parent == GTK_WIDGET (priv->list) ||
+           (GTK_IS_WIDGET (parent) && (gtk_widget_get_parent (parent) == GTK_WIDGET (priv->list)))) {
+    gtk_list_box_remove (priv->list, child);
+
+    if (!gtk_widget_get_first_child (GTK_WIDGET (priv->list)))
+      gtk_widget_add_css_class (GTK_WIDGET (self), "empty");
+  }
+  else {
+    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+  }
+}
+
 /**
  * adw_expander_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
  * @self: an expander row
@@ -326,8 +478,6 @@ adw_expander_row_new (void)
  * Gets the subtitle for @self.
  *
  * Returns: the subtitle for @self
- *
- * Since: 1.0
  */
 const char *
 adw_expander_row_get_subtitle (AdwExpanderRow *self)
@@ -350,8 +500,6 @@ adw_expander_row_get_subtitle (AdwExpanderRow *self)
  *
  * The subtitle is interpreted as Pango markup unless
  * [property@PreferencesRow:use-markup] is set to `FALSE`.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_subtitle (AdwExpanderRow *self,
@@ -374,7 +522,7 @@ adw_expander_row_set_subtitle (AdwExpanderRow *self,
  *
  * Returns: (nullable): the icon name for @self
  *
- * Since: 1.0
+ * Deprecated: 1.3: Use [method@ExpanderRow.add_prefix] to add an icon.
  */
 const char *
 adw_expander_row_get_icon_name (AdwExpanderRow *self)
@@ -395,7 +543,7 @@ adw_expander_row_get_icon_name (AdwExpanderRow *self)
  *
  * Sets the icon name for @self.
  *
- * Since: 1.0
+ * Deprecated: 1.3: Use [method@ExpanderRow.add_prefix] to add an icon.
  */
 void
 adw_expander_row_set_icon_name (AdwExpanderRow *self,
@@ -417,8 +565,6 @@ adw_expander_row_set_icon_name (AdwExpanderRow *self,
  * Gets whether @self is expanded.
  *
  * Returns: whether @self is expanded
- *
- * Since: 1.0
  */
 gboolean
 adw_expander_row_get_expanded (AdwExpanderRow *self)
@@ -438,8 +584,6 @@ adw_expander_row_get_expanded (AdwExpanderRow *self)
  * @expanded: whether to expand the row
  *
  * Sets whether @self is expanded.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_expanded (AdwExpanderRow *self,
@@ -473,8 +617,6 @@ adw_expander_row_set_expanded (AdwExpanderRow *self,
  * Gets whether the expansion of @self is enabled.
  *
  * Returns: whether the expansion of @self is enabled.
- *
- * Since: 1.0
  */
 gboolean
 adw_expander_row_get_enable_expansion (AdwExpanderRow *self)
@@ -494,8 +636,6 @@ adw_expander_row_get_enable_expansion (AdwExpanderRow *self)
  * @enable_expansion: whether to enable the expansion
  *
  * Sets whether the expansion of @self is enabled.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_enable_expansion (AdwExpanderRow *self,
@@ -526,8 +666,6 @@ adw_expander_row_set_enable_expansion (AdwExpanderRow *self,
  * Gets whether the switch enabling the expansion of @self is visible.
  *
  * Returns: whether the switch enabling the expansion is visible
- *
- * Since: 1.0
  */
 gboolean
 adw_expander_row_get_show_enable_switch (AdwExpanderRow *self)
@@ -547,8 +685,6 @@ adw_expander_row_get_show_enable_switch (AdwExpanderRow *self)
  * @show_enable_switch: whether to show the switch enabling the expansion
  *
  * Sets whether the switch enabling the expansion of @self is visible.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_show_enable_switch (AdwExpanderRow *self,
@@ -571,120 +707,99 @@ adw_expander_row_set_show_enable_switch (AdwExpanderRow *self,
 }
 
 /**
- * adw_expander_row_add_action:
+ * adw_expander_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
  * @self: an expander row
- * @widget: a widget
  *
- * Adds an action widget to @self.
+ * Gets the number of lines at the end of which the title label will be
+ * ellipsized.
  *
- * Since: 1.0
+ * Returns: the number of lines at the end of which the title label will be
+ *   ellipsized
+ *
+ * Since: 1.3
  */
-void
-adw_expander_row_add_action (AdwExpanderRow *self,
-                             GtkWidget      *widget)
+int
+adw_expander_row_get_title_lines (AdwExpanderRow *self)
 {
   AdwExpanderRowPrivate *priv;
 
-  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (self));
+  g_return_val_if_fail (ADW_IS_EXPANDER_ROW (self), 0);
 
   priv = adw_expander_row_get_instance_private (self);
 
-  gtk_box_prepend (priv->actions, widget);
-  gtk_widget_show (GTK_WIDGET (priv->actions));
+  return adw_action_row_get_title_lines (priv->action_row);
 }
 
 /**
- * adw_expander_row_add_prefix:
+ * adw_expander_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
  * @self: an expander row
- * @widget: a widget
+ * @title_lines: the number of lines at the end of which the title label will be ellipsized
  *
- * Adds a prefix widget to @self.
+ * Sets the number of lines at the end of which the title label will be
+ * ellipsized.
  *
- * Since: 1.0
+ * If the value is 0, the number of lines won't be limited.
+ *
+ * Since: 1.3
  */
 void
-adw_expander_row_add_prefix (AdwExpanderRow *self,
-                             GtkWidget      *widget)
+adw_expander_row_set_title_lines (AdwExpanderRow *self,
+                                  int             title_lines)
 {
   AdwExpanderRowPrivate *priv;
 
   g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (widget));
 
   priv = adw_expander_row_get_instance_private (self);
 
-  if (priv->prefixes == NULL) {
-    priv->prefixes = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12));
-    adw_action_row_add_prefix (ADW_ACTION_ROW (priv->action_row), GTK_WIDGET (priv->prefixes));
-  }
-  gtk_box_append (priv->prefixes, widget);
+  adw_action_row_set_title_lines (priv->action_row, title_lines);
 }
 
 /**
- * adw_expander_row_add_row:
+ * adw_expander_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
  * @self: an expander row
- * @child: a widget
  *
- * Adds a widget to @self.
+ * Gets the number of lines at the end of which the subtitle label will be
+ * ellipsized.
  *
- * The widget will appear in the expanding list below @self.
+ * Returns: the number of lines at the end of which the subtitle label will be
+ *   ellipsized
  *
- * Since: 1.0
+ * Since: 1.3
  */
-void
-adw_expander_row_add_row (AdwExpanderRow *self,
-                          GtkWidget      *child)
+int
+adw_expander_row_get_subtitle_lines (AdwExpanderRow *self)
 {
   AdwExpanderRowPrivate *priv;
 
-  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_val_if_fail (ADW_IS_EXPANDER_ROW (self), 0);
 
   priv = adw_expander_row_get_instance_private (self);
 
-  /* When constructing the widget, we want the box to be added as the child of
-   * the GtkListBoxRow, as an implementation detail.
-   */
-  gtk_list_box_append (priv->list, child);
-
-  gtk_widget_remove_css_class (GTK_WIDGET (self), "empty");
+  return adw_action_row_get_subtitle_lines (priv->action_row);
 }
 
 /**
- * adw_action_row_expander:
+ * adw_expander_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
  * @self: an expander row
- * @widget: the child to be removed
+ * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
  *
- * Removes a child from @self.
+ * Sets the number of lines at the end of which the subtitle label will be
+ * ellipsized.
  *
- * Since: 1.0
+ * If the value is 0, the number of lines won't be limited.
+ *
+ * Since: 1.3
  */
 void
-adw_expander_row_remove (AdwExpanderRow *self,
-                         GtkWidget      *child)
+adw_expander_row_set_subtitle_lines (AdwExpanderRow *self,
+                                     int             subtitle_lines)
 {
   AdwExpanderRowPrivate *priv;
-  GtkWidget *parent;
 
   g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
-  g_return_if_fail (GTK_IS_WIDGET (child));
 
   priv = adw_expander_row_get_instance_private (self);
 
-  parent = gtk_widget_get_parent (child);
-
-  if (parent == GTK_WIDGET (priv->actions))
-    gtk_box_remove (priv->actions, child);
-  else if (parent == GTK_WIDGET (priv->prefixes))
-    gtk_box_remove (priv->prefixes, child);
-  else if (parent == GTK_WIDGET (priv->list) ||
-           (GTK_IS_WIDGET (parent) && (gtk_widget_get_parent (parent) == GTK_WIDGET (priv->list)))) {
-    gtk_list_box_remove (priv->list, child);
-
-    if (!gtk_widget_get_first_child (GTK_WIDGET (priv->list)))
-      gtk_widget_add_css_class (GTK_WIDGET (self), "empty");
-  }
-  else
-    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+  adw_action_row_set_subtitle_lines (priv->action_row, subtitle_lines);
 }
