@@ -37,8 +37,6 @@
  *
  * `AdwPreferencesWindow` has a main CSS node with the name `window` and the
  * style class `.preferences`.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -138,9 +136,12 @@ make_comparable (const char        *src,
   GError *error = NULL;
 
   if (adw_preferences_row_get_use_markup (row)) {
-    pango_parse_markup (plaintext, -1, 0, NULL, &plaintext, NULL, &error);
+    char *parsed = NULL;
 
-    if (error) {
+    if (pango_parse_markup (plaintext, -1, 0, NULL, &parsed, NULL, &error)) {
+      g_free (plaintext);
+      plaintext = parsed;
+    } else {
       g_critical ("Couldn't parse markup: %s", error->message);
       g_clear_error (&error);
     }
@@ -553,8 +554,6 @@ adw_preferences_window_class_init (AdwPreferencesWindowClass *klass)
    * AdwViewStack:visible-page: (attributes org.gtk.Property.get=adw_preferences_window_get_visible_page org.gtk.Property.set=adw_preferences_window_set_visible_page)
    *
    * The currently visible page.
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_PAGE] =
     g_param_spec_object ("visible-page", NULL, NULL,
@@ -567,8 +566,6 @@ adw_preferences_window_class_init (AdwPreferencesWindowClass *klass)
    * The name of the currently visible page.
    *
    * See [property@ViewStack:visible-child].
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_PAGE_NAME] =
     g_param_spec_string ("visible-page-name", NULL, NULL,
@@ -579,8 +576,6 @@ adw_preferences_window_class_init (AdwPreferencesWindowClass *klass)
    * AdwPreferencesWindow:search-enabled: (attributes org.gtk.Property.get=adw_preferences_window_get_search_enabled org.gtk.Property.set=adw_preferences_window_set_search_enabled)
    *
    * Whether search is enabled.
-   *
-   * Since: 1.0
    */
   props[PROP_SEARCH_ENABLED] =
     g_param_spec_boolean ("search-enabled", NULL, NULL,
@@ -602,8 +597,6 @@ adw_preferences_window_class_init (AdwPreferencesWindowClass *klass)
    * <kbd>Alt</kbd>+<kbd>←</kbd> shortcut.
    *
    * For right-to-left locales, gestures and shortcuts are reversed.
-   *
-   * Since: 1.0
    */
   props[PROP_CAN_NAVIGATE_BACK] =
     g_param_spec_boolean ("can-navigate-back", NULL, NULL,
@@ -706,8 +699,6 @@ adw_preferences_window_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwPreferencesWindow`.
  *
  * Returns: the newly created `AdwPreferencesWindow`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_preferences_window_new (void)
@@ -715,6 +706,140 @@ adw_preferences_window_new (void)
   return g_object_new (ADW_TYPE_PREFERENCES_WINDOW, NULL);
 }
 
+/**
+ * adw_preferences_window_add:
+ * @self: a preferences window
+ * @page: the page to add
+ *
+ * Adds a preferences page to @self.
+ */
+void
+adw_preferences_window_add (AdwPreferencesWindow *self,
+                            AdwPreferencesPage   *page)
+{
+  AdwPreferencesWindowPrivate *priv;
+  AdwViewStackPage *stack_page;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (page));
+
+  priv = adw_preferences_window_get_instance_private (self);
+
+  stack_page = adw_view_stack_add_named (priv->pages_stack, GTK_WIDGET (page), adw_preferences_page_get_name (page));
+
+  g_object_bind_property (page, "icon-name", stack_page, "icon-name", G_BINDING_SYNC_CREATE);
+  g_object_bind_property (page, "title", stack_page, "title", G_BINDING_SYNC_CREATE);
+  g_object_bind_property (page, "use-underline", stack_page, "use-underline", G_BINDING_SYNC_CREATE);
+  g_object_bind_property (page, "name", stack_page, "name", G_BINDING_SYNC_CREATE);
+}
+
+/**
+ * adw_preferences_window_remove:
+ * @self: a preferences window
+ * @page: the page to remove
+ *
+ * Removes a page from @self.
+ */
+void
+adw_preferences_window_remove (AdwPreferencesWindow *self,
+                               AdwPreferencesPage   *page)
+{
+  AdwPreferencesWindowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (page));
+
+  priv = adw_preferences_window_get_instance_private (self);
+
+  if (gtk_widget_get_parent (GTK_WIDGET (page)) == GTK_WIDGET (priv->pages_stack))
+    adw_view_stack_remove (priv->pages_stack, GTK_WIDGET (page));
+  else
+    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, page);
+}
+
+/**
+ * adw_preferences_window_get_visible_page:
+ * @self: a preferences window
+ *
+ * Gets the currently visible page of @self.
+ *
+ * Returns: (transfer none) (nullable): the visible page
+ */
+AdwPreferencesPage *
+adw_preferences_window_get_visible_page (AdwPreferencesWindow *self)
+{
+  AdwPreferencesWindowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_WINDOW (self), NULL);
+
+  priv = adw_preferences_window_get_instance_private (self);
+
+  return ADW_PREFERENCES_PAGE (adw_view_stack_get_visible_child (priv->pages_stack));
+}
+
+/**
+ * adw_preferences_window_set_visible_page:
+ * @self: a preferences window
+ * @page: a page of @self
+ *
+ * Makes @page the visible page of @self.
+ */
+void
+adw_preferences_window_set_visible_page (AdwPreferencesWindow *self,
+                                         AdwPreferencesPage   *page)
+{
+  AdwPreferencesWindowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (page));
+
+  priv = adw_preferences_window_get_instance_private (self);
+
+  adw_view_stack_set_visible_child (priv->pages_stack, GTK_WIDGET (page));
+}
+
+/**
+ * adw_preferences_window_get_visible_page_name:
+ * @self: a preferences window
+ *
+ * Gets the name of currently visible page of @self.
+ *
+ * Returns: (transfer none) (nullable): the name of the visible page
+ */
+const char *
+adw_preferences_window_get_visible_page_name (AdwPreferencesWindow *self)
+{
+  AdwPreferencesWindowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_WINDOW (self), NULL);
+
+  priv = adw_preferences_window_get_instance_private (self);
+
+  return adw_view_stack_get_visible_child_name (priv->pages_stack);
+}
+
+/**
+ * adw_preferences_window_set_visible_page_name:
+ * @self: a preferences window
+ * @name: the name of the page to make visible
+ *
+ * Makes the page with the given name visible.
+ *
+ * See [property@ViewStack:visible-child].
+ */
+void
+adw_preferences_window_set_visible_page_name (AdwPreferencesWindow *self,
+                                              const char           *name)
+{
+  AdwPreferencesWindowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
+
+  priv = adw_preferences_window_get_instance_private (self);
+
+  adw_view_stack_set_visible_child_name (priv->pages_stack, name);
+}
+
 /**
  * adw_preferences_window_get_search_enabled: (attributes org.gtk.Method.get_property=search-enabled)
  * @self: a preferences window
@@ -722,8 +847,6 @@ adw_preferences_window_new (void)
  * Gets whether search is enabled for @self.
  *
  * Returns: whether search is enabled for @self.
- *
- * Since: 1.0
  */
 gboolean
 adw_preferences_window_get_search_enabled (AdwPreferencesWindow *self)
@@ -743,8 +866,6 @@ adw_preferences_window_get_search_enabled (AdwPreferencesWindow *self)
  * @search_enabled: whether search is enabled
  *
  * Sets whether search is enabled for @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_window_set_search_enabled (AdwPreferencesWindow *self,
@@ -780,7 +901,16 @@ adw_preferences_window_set_search_enabled (AdwPreferencesWindow *self,
  *
  * Sets whether gestures and shortcuts for closing subpages are enabled.
  *
- * Since: 1.0
+ * The supported gestures are:
+ *
+ * - One-finger swipe on touchscreens
+ * - Horizontal scrolling on touchpads (usually two-finger swipe)
+ * - Back mouse button
+ *
+ * The keyboard back key is also supported, as well as the
+ * <kbd>Alt</kbd>+<kbd>←</kbd> shortcut.
+ *
+ * For right-to-left locales, gestures and shortcuts are reversed.
  */
 void
 adw_preferences_window_set_can_navigate_back (AdwPreferencesWindow *self,
@@ -809,8 +939,6 @@ adw_preferences_window_set_can_navigate_back (AdwPreferencesWindow *self,
  * Gets whether gestures and shortcuts for closing subpages are enabled.
  *
  * Returns: whether gestures and shortcuts are enabled.
- *
- * Since: 1.0
  */
 gboolean
 adw_preferences_window_get_can_navigate_back (AdwPreferencesWindow *self)
@@ -833,8 +961,6 @@ adw_preferences_window_get_can_navigate_back (AdwPreferencesWindow *self)
  *
  * The transition can be cancelled by the user, in which case visible child will
  * change back to the previously visible child.
- *
- * Since: 1.0
  */
 void
 adw_preferences_window_present_subpage (AdwPreferencesWindow *self,
@@ -868,8 +994,6 @@ adw_preferences_window_present_subpage (AdwPreferencesWindow *self,
  * Closes the current subpage.
  *
  * If there is no presented subpage, this does nothing.
- *
- * Since: 1.0
  */
 void
 adw_preferences_window_close_subpage (AdwPreferencesWindow *self)
@@ -886,150 +1010,6 @@ adw_preferences_window_close_subpage (AdwPreferencesWindow *self)
   adw_leaflet_set_visible_child (priv->subpages_leaflet, priv->preferences);
 }
 
-/**
- * adw_preferences_window_add:
- * @self: a preferences window
- * @page: the page to add
- *
- * Adds a preferences page to @self.
- *
- * Since: 1.0
- */
-void
-adw_preferences_window_add (AdwPreferencesWindow *self,
-                            AdwPreferencesPage   *page)
-{
-  AdwPreferencesWindowPrivate *priv;
-  AdwViewStackPage *stack_page;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
-  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (page));
-
-  priv = adw_preferences_window_get_instance_private (self);
-
-  stack_page = adw_view_stack_add_named (priv->pages_stack, GTK_WIDGET (page), adw_preferences_page_get_name (page));
-
-  g_object_bind_property (page, "icon-name", stack_page, "icon-name", G_BINDING_SYNC_CREATE);
-  g_object_bind_property (page, "title", stack_page, "title", G_BINDING_SYNC_CREATE);
-  g_object_bind_property (page, "use-underline", stack_page, "use-underline", G_BINDING_SYNC_CREATE);
-  g_object_bind_property (page, "name", stack_page, "name", G_BINDING_SYNC_CREATE);
-}
-
-/**
- * adw_preferences_window_remove:
- * @self: a preferences window
- * @page: the page to remove
- *
- * Removes a page from @self.
- *
- * Since: 1.0
- */
-void
-adw_preferences_window_remove (AdwPreferencesWindow *self,
-                               AdwPreferencesPage   *page)
-{
-  AdwPreferencesWindowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
-  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (page));
-
-  priv = adw_preferences_window_get_instance_private (self);
-
-  if (gtk_widget_get_parent (GTK_WIDGET (page)) == GTK_WIDGET (priv->pages_stack))
-    adw_view_stack_remove (priv->pages_stack, GTK_WIDGET (page));
-  else
-    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, page);
-}
-
-/**
- * adw_preferences_window_get_visible_page:
- * @self: a preferences window
- *
- * Gets the currently visible page of @self.
- *
- * Returns: (transfer none) (nullable): the visible page
- *
- * Since: 1.0
- */
-AdwPreferencesPage *
-adw_preferences_window_get_visible_page (AdwPreferencesWindow *self)
-{
-  AdwPreferencesWindowPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_PREFERENCES_WINDOW (self), NULL);
-
-  priv = adw_preferences_window_get_instance_private (self);
-
-  return ADW_PREFERENCES_PAGE (adw_view_stack_get_visible_child (priv->pages_stack));
-}
-
-/**
- * adw_preferences_window_set_visible_page:
- * @self: a preferences window
- * @page: a page of @self
- *
- * Makes @page the visible page of @self.
- *
- * Since: 1.0
- */
-void
-adw_preferences_window_set_visible_page (AdwPreferencesWindow *self,
-                                         AdwPreferencesPage   *page)
-{
-  AdwPreferencesWindowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
-  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (page));
-
-  priv = adw_preferences_window_get_instance_private (self);
-
-  adw_view_stack_set_visible_child (priv->pages_stack, GTK_WIDGET (page));
-}
-
-/**
- * adw_preferences_window_get_visible_page_name:
- * @self: a preferences window
- *
- * Gets the name of currently visible page of @self.
- *
- * Returns: (transfer none) (nullable): the name of the visible page
- *
- * Since: 1.0
- */
-const char *
-adw_preferences_window_get_visible_page_name (AdwPreferencesWindow *self)
-{
-  AdwPreferencesWindowPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_PREFERENCES_WINDOW (self), NULL);
-
-  priv = adw_preferences_window_get_instance_private (self);
-
-  return adw_view_stack_get_visible_child_name (priv->pages_stack);
-}
-
-/**
- * adw_preferences_window_set_visible_page_name:
- * @self: a preferences window
- * @name: the name of the page to make visible
- *
- * Makes the page with the given name visible.
- *
- * Since: 1.0
- */
-void
-adw_preferences_window_set_visible_page_name (AdwPreferencesWindow *self,
-                                              const char           *name)
-{
-  AdwPreferencesWindowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_WINDOW (self));
-
-  priv = adw_preferences_window_get_instance_private (self);
-
-  adw_view_stack_set_visible_child_name (priv->pages_stack, name);
-}
-
 /**
  * adw_preferences_window_add_toast:
  * @self: a preferences window
@@ -1038,8 +1018,6 @@ adw_preferences_window_set_visible_page_name (AdwPreferencesWindow *self,
  * Displays @toast.
  *
  * See [method@ToastOverlay.add_toast].
- *
- * Since: 1.0
  */
 void
 adw_preferences_window_add_toast (AdwPreferencesWindow *self,
