@@ -10,7 +10,6 @@
 
 #include "adw-style-manager-private.h"
 
-#include "adw-macros-private.h"
 #include "adw-settings-private.h"
 #include <gtk/gtk.h>
 
@@ -29,8 +28,6 @@
  * @ADW_COLOR_SCHEME_FORCE_DARK: Always use dark appearance.
  *
  * Application color schemes for [property@StyleManager:color-scheme].
- *
- * Since: 1.0
  */
 
 /**
@@ -44,8 +41,6 @@
  * It allows to set the color scheme via the
  * [property@StyleManager:color-scheme] property, and to query the current
  * appearance, as well as whether a system-wide color scheme preference exists.
- *
- * Since: 1.0
  */
 
 struct _AdwStyleManager
@@ -387,8 +382,6 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    *
    * The display will be `NULL` for the style manager returned by
    * [func@StyleManager.get_default].
-   *
-   * Since: 1.0
    */
   props[PROP_DISPLAY] =
     g_param_spec_object ("display", NULL, NULL,
@@ -429,8 +422,6 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * The [property@StyleManager:system-supports-color-schemes] property can be
    * used to check if the current environment provides a color scheme
    * preference.
-   *
-   * Since: 1.0
    */
   props[PROP_COLOR_SCHEME] =
     g_param_spec_enum ("color-scheme", NULL, NULL,
@@ -448,8 +439,6 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * separate appearance switcher if it's set to `FALSE`.
    *
    * See [property@StyleManager:color-scheme].
-   *
-   * Since: 1.0
    */
   props[PROP_SYSTEM_SUPPORTS_COLOR_SCHEMES] =
     g_param_spec_boolean ("system-supports-color-schemes", NULL, NULL,
@@ -463,8 +452,6 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    *
    * This property can be used to query the current appearance, as requested via
    * [property@StyleManager:color-scheme].
-   *
-   * Since: 1.0
    */
   props[PROP_DARK] =
     g_param_spec_boolean ("dark", NULL, NULL,
@@ -477,8 +464,6 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Whether the application is using high contrast appearance.
    *
    * This cannot be overridden by applications.
-   *
-   * Since: 1.0
    */
   props[PROP_HIGH_CONTRAST] =
     g_param_spec_boolean ("high-contrast", NULL, NULL,
@@ -534,8 +519,6 @@ adw_style_manager_ensure (void)
  * See [func@StyleManager.get_for_display].
  *
  * Returns: (transfer none): the default style manager
- *
- * Since: 1.0
  */
 AdwStyleManager *
 adw_style_manager_get_default (void)
@@ -558,8 +541,6 @@ adw_style_manager_get_default (void)
  * Most applications should use [func@StyleManager.get_default] instead.
  *
  * Returns: (transfer none): the style manager for @display
- *
- * Since: 1.0
  */
 AdwStyleManager *
 adw_style_manager_get_for_display (GdkDisplay *display)
@@ -584,8 +565,6 @@ adw_style_manager_get_for_display (GdkDisplay *display)
  * [func@StyleManager.get_default].
  *
  * Returns: (transfer none): (nullable): the display
- *
- * Since: 1.0
  */
 GdkDisplay *
 adw_style_manager_get_display (AdwStyleManager *self)
@@ -602,8 +581,6 @@ adw_style_manager_get_display (AdwStyleManager *self)
  * Gets the requested application color scheme.
  *
  * Returns: the color scheme
- *
- * Since: 1.0
  */
 AdwColorScheme
 adw_style_manager_get_color_scheme (AdwStyleManager *self)
@@ -625,7 +602,30 @@ adw_style_manager_get_color_scheme (AdwStyleManager *self)
  * [property@StyleManager:dark] property can be used to query the current
  * effective appearance.
  *
- * Since: 1.0
+ * The `ADW_COLOR_SCHEME_PREFER_LIGHT` color scheme results in the application
+ * using light appearance unless the system prefers dark colors. This is the
+ * default value.
+ *
+ * The `ADW_COLOR_SCHEME_PREFER_DARK` color scheme results in the application
+ * using dark appearance, but can still switch to the light appearance if the
+ * system can prefers it, for example, when the high contrast preference is
+ * enabled.
+ *
+ * The `ADW_COLOR_SCHEME_FORCE_LIGHT` and `ADW_COLOR_SCHEME_FORCE_DARK` values
+ * ignore the system preference entirely. They are useful if the application
+ * wants to match its UI to its content or to provide a separate color scheme
+ * switcher.
+ *
+ * If a per-[class@Gdk.Display] style manager has its color scheme set to
+ * `ADW_COLOR_SCHEME_DEFAULT`, it will inherit the color scheme from the
+ * default style manager.
+ *
+ * For the default style manager, `ADW_COLOR_SCHEME_DEFAULT` is equivalent to
+ * `ADW_COLOR_SCHEME_PREFER_LIGHT`.
+ *
+ * The [property@StyleManager:system-supports-color-schemes] property can be
+ * used to check if the current environment provides a color scheme
+ * preference.
  */
 void
 adw_style_manager_set_color_scheme (AdwStyleManager *self,
@@ -664,9 +664,11 @@ adw_style_manager_set_color_scheme (AdwStyleManager *self,
  *
  * Gets whether the system supports color schemes.
  *
- * Returns: whether the system supports color schemes
+ * This can be used to check if the current environment provides a color scheme
+ * preference. For example, applications might want to show a separate
+ * appearance switcher if it's set to `FALSE`.
  *
- * Since: 1.0
+ * Returns: whether the system supports color schemes
  */
 gboolean
 adw_style_manager_get_system_supports_color_schemes (AdwStyleManager *self)
@@ -682,9 +684,10 @@ adw_style_manager_get_system_supports_color_schemes (AdwStyleManager *self)
  *
  * Gets whether the application is using dark appearance.
  *
- * Returns: whether the application is using dark appearance
+ * This can be used to query the current appearance, as requested via
+ * [property@StyleManager:color-scheme].
  *
- * Since: 1.0
+ * Returns: whether the application is using dark appearance
  */
 gboolean
 adw_style_manager_get_dark (AdwStyleManager *self)
@@ -700,9 +703,9 @@ adw_style_manager_get_dark (AdwStyleManager *self)
  *
  * Gets whether the application is using high contrast appearance.
  *
- * Returns: whether the application is using high contrast appearance
+ * This cannot be overridden by applications.
  *
- * Since: 1.0
+ * Returns: whether the application is using high contrast appearance
  */
 gboolean
 adw_style_manager_get_high_contrast (AdwStyleManager *self)
