@@ -7,8 +7,8 @@
 #include "config.h"
 
 #include "adw-swipe-tracker-private.h"
+
 #include "adw-navigation-direction.h"
-#include "adw-macros-private.h"
 
 #include <math.h>
 
@@ -28,12 +28,6 @@
 #define DRAG_THRESHOLD_DISTANCE 16
 #define EPSILON 0.005
 
-#if GTK_CHECK_VERSION (4, 7, 0)
-#define SCROLL_MULTIPLIER 1
-#else
-#define SCROLL_MULTIPLIER 10
-#endif
-
 #define SIGN(x) ((x) > 0.0 ? 1.0 : ((x) < 0.0 ? -1.0 : 0.0))
 
 /**
@@ -49,8 +43,6 @@
  * property. If they expect to use horizontal orientation,
  * [property@SwipeTracker:reversed] can be used for supporting RTL text
  * direction.
- *
- * Since: 1.0
  */
 
 typedef enum {
@@ -291,7 +283,7 @@ find_next_point (double *points,
   guint i;
 
   for (i = 0; i < n; i++)
-    if (points[i] >= pos)
+    if (G_APPROX_VALUE (points[i], pos, DBL_EPSILON) || points[i] > pos)
       return i;
 
   return -1;
@@ -305,7 +297,7 @@ find_previous_point (double *points,
   int i;
 
   for (i = n - 1; i >= 0; i--)
-    if (points[i] <= pos)
+    if (G_APPROX_VALUE (points[i], pos, DBL_EPSILON) || points[i] < pos)
       return i;
 
   return -1;
@@ -502,8 +494,10 @@ is_in_swipe_area (AdwSwipeTracker        *self,
 
   adw_swipeable_get_swipe_area (self->swipeable, direction, is_drag, &rect);
 
-  return x >= rect.x && x < rect.x + rect.width &&
-         y >= rect.y && y < rect.y + rect.height;
+  return (G_APPROX_VALUE (x, rect.x, DBL_EPSILON) || x > rect.x) &&
+         x < rect.x + rect.width &&
+         (G_APPROX_VALUE (y, rect.y, DBL_EPSILON) || y > rect.y) &&
+         y < rect.y + rect.height;
 }
 
 static void
@@ -589,10 +583,13 @@ drag_update_cb (AdwSwipeTracker *self,
     get_range (self, &first_point, &last_point);
 
     drag_distance = sqrt (offset_x * offset_x + offset_y * offset_y);
-    is_overshooting = (offset < 0 && self->progress <= first_point) ||
-                      (offset > 0 && self->progress >= last_point);
+    is_overshooting = (offset < 0 && (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
+                                      self->progress < first_point)) ||
+                      (offset > 0 && (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
+                                      self->progress > last_point));
 
-    if (drag_distance >= DRAG_THRESHOLD_DISTANCE) {
+    if (G_APPROX_VALUE (drag_distance, DRAG_THRESHOLD_DISTANCE, DBL_EPSILON) ||
+        drag_distance > DRAG_THRESHOLD_DISTANCE) {
       double start_x, start_y;
       AdwNavigationDirection direction;
 
@@ -722,10 +719,14 @@ handle_scroll_event (AdwSwipeTracker *self,
 
     get_range (self, &first_point, &last_point);
 
-    is_overshooting = (delta < 0 && self->progress <= first_point) ||
-                      (delta > 0 && self->progress >= last_point);
+    is_overshooting = (delta < 0 &&
+                       (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
+                        self->progress < first_point)) ||
+                      (delta > 0 &&
+                       (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
+                        self->progress > last_point));
 
-    append_to_history (self, delta * SCROLL_MULTIPLIER, time);
+    append_to_history (self, delta, time);
 
     if (!is_overshooting)
       gesture_begin (self);
@@ -737,9 +738,9 @@ handle_scroll_event (AdwSwipeTracker *self,
     if (gdk_scroll_event_is_stop (event)) {
       gesture_end (self, distance, time, TRUE);
     } else {
-      append_to_history (self, delta * SCROLL_MULTIPLIER, time);
+      append_to_history (self, delta, time);
 
-      gesture_update (self, delta / distance * SCROLL_MULTIPLIER, time);
+      gesture_update (self, delta / distance, time);
       return GDK_EVENT_STOP;
     }
   }
@@ -1023,8 +1024,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * AdwSwipeTracker:swipeable: (attributes org.gtk.Property.get=adw_swipe_tracker_get_swipeable)
    *
    * The widget the swipe tracker is attached to.
-   *
-   * Since: 1.0
    */
   props[PROP_SWIPEABLE] =
     g_param_spec_object ("swipeable", NULL, NULL,
@@ -1038,8 +1037,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * When it's not enabled, no events will be processed. Usually widgets will
    * want to expose this via a property.
-   *
-   * Since: 1.0
    */
   props[PROP_ENABLED] =
     g_param_spec_boolean ("enabled", NULL, NULL,
@@ -1053,8 +1050,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * If the swipe tracker is horizontal, it can be used for supporting RTL text
    * direction.
-   *
-   * Since: 1.0
    */
   props[PROP_REVERSED] =
     g_param_spec_boolean ("reversed", NULL, NULL,
@@ -1065,8 +1060,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * AdwSwipeTracker:allow-mouse-drag: (attributes org.gtk.Property.get=adw_swipe_tracker_get_allow_mouse_drag org.gtk.Property.set=adw_swipe_tracker_set_allow_mouse_drag)
    *
    * Whether to allow dragging with mouse pointer.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_MOUSE_DRAG] =
     g_param_spec_boolean ("allow-mouse-drag", NULL, NULL,
@@ -1080,8 +1073,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * If the value is `FALSE`, each swipe can only move to the adjacent snap
    * points.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_LONG_SWIPES] =
     g_param_spec_boolean ("allow-long-swipes", NULL, NULL,
@@ -1103,8 +1094,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * The @direction value can be used to restrict the swipe to a certain
    * direction.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_PREPARE] =
     g_signal_new ("prepare",
@@ -1121,8 +1110,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * This signal is emitted right before a swipe will be started, after the
    * drag threshold has been passed.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_BEGIN_SWIPE] =
     g_signal_new ("begin-swipe",
@@ -1139,8 +1126,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * @progress: the current animation progress value
    *
    * This signal is emitted every time the progress value changes.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_UPDATE_SWIPE] =
     g_signal_new ("update-swipe",
@@ -1164,8 +1149,6 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * value to @to with an animation using @velocity as the initial velocity,
    * provided in pixels per second. [class@SpringAnimation] is usually a good
    * fit for this.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_END_SWIPE] =
     g_signal_new ("end-swipe",
@@ -1195,8 +1178,6 @@ adw_swipe_tracker_init (AdwSwipeTracker *self)
  * Creates a new `AdwSwipeTracker` for @widget.
  *
  * Returns: the newly created `AdwSwipeTracker`
- *
- * Since: 1.0
  */
 AdwSwipeTracker *
 adw_swipe_tracker_new (AdwSwipeable *swipeable)
@@ -1215,8 +1196,6 @@ adw_swipe_tracker_new (AdwSwipeable *swipeable)
  * Get the widget @self is attached to.
  *
  * Returns: (transfer none): the swipeable widget
- *
- * Since: 1.0
  */
 AdwSwipeable *
 adw_swipe_tracker_get_swipeable (AdwSwipeTracker *self)
@@ -1233,8 +1212,6 @@ adw_swipe_tracker_get_swipeable (AdwSwipeTracker *self)
  * Gets whether @self is enabled.
  *
  * Returns: whether @self is enabled
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_enabled (AdwSwipeTracker *self)
@@ -1251,7 +1228,8 @@ adw_swipe_tracker_get_enabled (AdwSwipeTracker *self)
  *
  * Sets whether @self is enabled.
  *
- * Since: 1.0
+ * When it's not enabled, no events will be processed. Usually widgets will want
+ * to expose this via a property.
  */
 void
 adw_swipe_tracker_set_enabled (AdwSwipeTracker *self,
@@ -1281,8 +1259,6 @@ adw_swipe_tracker_set_enabled (AdwSwipeTracker *self,
  * Gets whether @self is reversing the swipe direction.
  *
  * Returns: whether the direction is reversed
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_reversed (AdwSwipeTracker *self)
@@ -1299,7 +1275,8 @@ adw_swipe_tracker_get_reversed (AdwSwipeTracker *self)
  *
  * Sets whether to reverse the swipe direction.
  *
- * Since: 1.0
+ * If the swipe tracker is horizontal, it can be used for supporting RTL text
+ * direction.
  */
 void
 adw_swipe_tracker_set_reversed (AdwSwipeTracker *self,
@@ -1323,8 +1300,6 @@ adw_swipe_tracker_set_reversed (AdwSwipeTracker *self,
  * Gets whether @self can be dragged with mouse pointer.
  *
  * Returns: whether mouse dragging is allowed
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_allow_mouse_drag (AdwSwipeTracker *self)
@@ -1340,8 +1315,6 @@ adw_swipe_tracker_get_allow_mouse_drag (AdwSwipeTracker *self)
  * @allow_mouse_drag: whether to allow mouse dragging
  *
  * Sets whether @self can be dragged with mouse pointer.
- *
- * Since: 1.0
  */
 void
 adw_swipe_tracker_set_allow_mouse_drag (AdwSwipeTracker *self,
@@ -1368,8 +1341,6 @@ adw_swipe_tracker_set_allow_mouse_drag (AdwSwipeTracker *self,
  * Gets whether to allow swiping for more than one snap point at a time.
  *
  * Returns: whether long swipes are allowed
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_allow_long_swipes (AdwSwipeTracker *self)
@@ -1386,7 +1357,8 @@ adw_swipe_tracker_get_allow_long_swipes (AdwSwipeTracker *self)
  *
  * Sets whether to allow swiping for more than one snap point at a time.
  *
- * Since: 1.0
+ * If the value is `FALSE`, each swipe can only move to the adjacent snap
+ * points.
  */
 void
 adw_swipe_tracker_set_allow_long_swipes (AdwSwipeTracker *self,
@@ -1413,8 +1385,6 @@ adw_swipe_tracker_set_allow_long_swipes (AdwSwipeTracker *self,
  *
  * This can be used to adjust the current position if snap points move during
  * the gesture.
- *
- * Since: 1.0
  */
 void
 adw_swipe_tracker_shift_position (AdwSwipeTracker *self,
