@@ -16,7 +16,6 @@
 
 #include "adw-animation-util.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -72,15 +71,16 @@
  *
  * `AdwViewStack` has a single CSS node named `stack`.
  *
- * Since: 1.0
+ * ## Accessibility
+ *
+ * `AdwViewStack` uses the `GTK_ACCESSIBLE_ROLE_TAB_PANEL` for the stack pages
+ * which are the accessible parent objects of the child widgets.
  */
 
 /**
  * AdwViewStackPage:
  *
  * An auxiliary class used by [class@ViewStack].
- *
- * Since: 1.0
  */
 
 #define OPPOSITE_ORIENTATION(_orientation) (1 - (_orientation))
@@ -105,24 +105,31 @@ struct _AdwViewStackPage {
   guint badge_number;
   GtkWidget *last_focus;
 
+  GtkATContext *at_context;
+  AdwViewStackPage *next_page;
+
   bool needs_attention;
   bool visible;
   bool use_underline;
 };
 
-G_DEFINE_FINAL_TYPE (AdwViewStackPage, adw_view_stack_page, G_TYPE_OBJECT)
+static void adw_view_stack_page_accessible_init (GtkAccessibleInterface *iface);
+
+G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStackPage, adw_view_stack_page, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE, adw_view_stack_page_accessible_init))
 
 enum {
   PAGE_PROP_0,
   PAGE_PROP_CHILD,
   PAGE_PROP_NAME,
   PAGE_PROP_TITLE,
+  PAGE_PROP_USE_UNDERLINE,
   PAGE_PROP_ICON_NAME,
   PAGE_PROP_NEEDS_ATTENTION,
   PAGE_PROP_BADGE_NUMBER,
   PAGE_PROP_VISIBLE,
-  PAGE_PROP_USE_UNDERLINE,
-  LAST_PAGE_PROP
+  LAST_PAGE_PROP,
+  PAGE_PROP_ACCESSIBLE_ROLE
 };
 
 static GParamSpec *page_props[LAST_PAGE_PROP];
@@ -142,9 +149,11 @@ struct _AdwViewStack {
 static GParamSpec *props[LAST_PROP];
 
 static void adw_view_stack_buildable_init (GtkBuildableIface *iface);
+static void adw_view_stack_accessible_init (GtkAccessibleInterface *iface);
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStack, adw_view_stack, GTK_TYPE_WIDGET,
-                               G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, adw_view_stack_buildable_init))
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, adw_view_stack_buildable_init)
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE, adw_view_stack_accessible_init))
 
 static GtkBuildableIface *parent_buildable_iface;
 
@@ -166,6 +175,9 @@ adw_view_stack_page_get_property (GObject      *object,
   case PAGE_PROP_TITLE:
     g_value_set_string (value, adw_view_stack_page_get_title (self));
     break;
+  case PAGE_PROP_USE_UNDERLINE:
+    g_value_set_boolean (value, adw_view_stack_page_get_use_underline (self));
+    break;
   case PAGE_PROP_ICON_NAME:
     g_value_set_string (value, adw_view_stack_page_get_icon_name (self));
     break;
@@ -178,8 +190,8 @@ adw_view_stack_page_get_property (GObject      *object,
   case PAGE_PROP_VISIBLE:
     g_value_set_boolean (value, adw_view_stack_page_get_visible (self));
     break;
-  case PAGE_PROP_USE_UNDERLINE:
-    g_value_set_boolean (value, adw_view_stack_page_get_use_underline (self));
+  case PAGE_PROP_ACCESSIBLE_ROLE:
+    g_value_set_enum (value, GTK_ACCESSIBLE_ROLE_TAB_PANEL);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -205,6 +217,9 @@ adw_view_stack_page_set_property (GObject      *object,
   case PAGE_PROP_TITLE:
     adw_view_stack_page_set_title (self, g_value_get_string (value));
     break;
+  case PAGE_PROP_USE_UNDERLINE:
+    adw_view_stack_page_set_use_underline (self, g_value_get_boolean (value));
+    break;
   case PAGE_PROP_ICON_NAME:
     adw_view_stack_page_set_icon_name (self, g_value_get_string (value));
     break;
@@ -217,8 +232,7 @@ adw_view_stack_page_set_property (GObject      *object,
   case PAGE_PROP_VISIBLE:
     adw_view_stack_page_set_visible (self, g_value_get_boolean (value));
     break;
-  case PAGE_PROP_USE_UNDERLINE:
-    adw_view_stack_page_set_use_underline (self, g_value_get_boolean (value));
+  case PAGE_PROP_ACCESSIBLE_ROLE:
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -226,6 +240,16 @@ adw_view_stack_page_set_property (GObject      *object,
   }
 }
 
+static void
+adw_view_stack_page_dispose (GObject *object)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (object);
+
+  g_clear_object (&self->at_context);
+
+  G_OBJECT_CLASS (adw_view_stack_page_parent_class)->dispose (object);
+}
+
 static void
 adw_view_stack_page_finalize (GObject *object)
 {
@@ -248,6 +272,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  object_class->dispose = adw_view_stack_page_dispose;
   object_class->finalize = adw_view_stack_page_finalize;
   object_class->get_property = adw_view_stack_page_get_property;
   object_class->set_property = adw_view_stack_page_set_property;
@@ -255,9 +280,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
   /**
    * AdwViewStackPage:child: (attributes org.gtk.Property.get=adw_view_stack_page_get_child)
    *
-   * The child of the page.
-   *
-   * Since: 1.0
+   * The stack child to which the page belongs.
    */
   page_props[PAGE_PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -268,8 +291,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:name: (attributes org.gtk.Property.get=adw_view_stack_page_get_name org.gtk.Property.set=adw_view_stack_page_set_name)
    *
    * The name of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_NAME] =
     g_param_spec_string ("name", NULL, NULL,
@@ -280,20 +301,26 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:title: (attributes org.gtk.Property.get=adw_view_stack_page_get_title org.gtk.Property.set=adw_view_stack_page_set_title)
    *
    * The title of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
+   *
+   * Whether an embedded underline in the title indicates a mnemonic.
+   */
+  page_props[PAGE_PROP_USE_UNDERLINE] =
+    g_param_spec_boolean ("use-underline", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * AdwViewStackPage:icon-name: (attributes org.gtk.Property.get=adw_view_stack_page_get_icon_name org.gtk.Property.set=adw_view_stack_page_set_icon_name)
    *
    * The icon name of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -306,8 +333,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * Whether the page requires the user attention.
    *
    * [class@ViewSwitcher] will display it as a dot next to the page icon.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_NEEDS_ATTENTION] =
     g_param_spec_boolean ("needs-attention", NULL, NULL,
@@ -317,14 +342,12 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
   /**
    * AdwViewStackPage:badge-number: (attributes org.gtk.Property.get=adw_view_stack_page_get_badge_number org.gtk.Property.set=adw_view_stack_page_set_badge_number)
    *
-   * A number associated with the page.
+   * The badge number for this page.
    *
    * [class@ViewSwitcher] can display it as a badge next to the page icon. It is
    * commonly used to display a number of unread items within the page.
    *
    * It can be used together with [property@ViewStack{age}:needs-attention].
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_BADGE_NUMBER] =
     g_param_spec_uint ("badge-number", NULL, NULL,
@@ -338,27 +361,15 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    *
    * This is independent from the [property@Gtk.Widget:visible] property of
    * [property@ViewStackPage:child].
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_VISIBLE] =
     g_param_spec_boolean ("visible", NULL, NULL,
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
-   *
-   * Whether an embedded underline in the title indicates a mnemonic.
-   *
-   * Since: 1.0
-   */
-  page_props[PAGE_PROP_USE_UNDERLINE] =
-    g_param_spec_boolean ("use-underline", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
   g_object_class_install_properties (object_class, LAST_PAGE_PROP, page_props);
+
+  g_object_class_override_property (object_class, PAGE_PROP_ACCESSIBLE_ROLE, "accessible-role");
 }
 
 static void
@@ -367,6 +378,89 @@ adw_view_stack_page_init (AdwViewStackPage *self)
   self->visible = TRUE;
 }
 
+static GtkATContext *
+adw_view_stack_page_accessible_get_at_context (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (self->at_context == NULL) {
+    GtkAccessibleRole role = GTK_ACCESSIBLE_ROLE_TAB_PANEL;
+    GdkDisplay *display;
+
+    if (self->widget != NULL)
+      display = gtk_widget_get_display (self->widget);
+    else
+      display = gdk_display_get_default ();
+
+    self->at_context = gtk_at_context_create (role, accessible, display);
+  }
+
+  return self->at_context;
+}
+
+static gboolean
+adw_view_stack_page_accessible_get_platform_state (GtkAccessible              *self,
+                                                   GtkAccessiblePlatformState  state)
+{
+  return FALSE;
+}
+
+static GtkAccessible *
+adw_view_stack_page_accessible_get_accessible_parent (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (self->widget)
+    return GTK_ACCESSIBLE (gtk_widget_get_parent (self->widget));
+
+  return NULL;
+}
+
+static GtkAccessible *
+adw_view_stack_page_accessible_get_first_accessible_child (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (self->widget)
+    return GTK_ACCESSIBLE (self->widget);
+
+  return NULL;
+}
+
+static GtkAccessible *
+adw_view_stack_page_accessible_get_next_accessible_sibling (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  return GTK_ACCESSIBLE (self->next_page);
+}
+
+static gboolean
+adw_view_stack_page_accessible_get_bounds (GtkAccessible *accessible,
+                                           int           *x,
+                                           int           *y,
+                                           int           *width,
+                                           int           *height)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (self->widget)
+    return gtk_accessible_get_bounds (GTK_ACCESSIBLE (self->widget), x, y, width, height);
+
+  return FALSE;
+}
+
+static void
+adw_view_stack_page_accessible_init (GtkAccessibleInterface *iface)
+{
+  iface->get_at_context = adw_view_stack_page_accessible_get_at_context;
+  iface->get_platform_state = adw_view_stack_page_accessible_get_platform_state;
+  iface->get_accessible_parent = adw_view_stack_page_accessible_get_accessible_parent;
+  iface->get_first_accessible_child = adw_view_stack_page_accessible_get_first_accessible_child;
+  iface->get_next_accessible_sibling = adw_view_stack_page_accessible_get_next_accessible_sibling;
+  iface->get_bounds = adw_view_stack_page_accessible_get_bounds;
+}
+
 #define ADW_TYPE_VIEW_STACK_PAGES (adw_view_stack_pages_get_type ())
 
 G_DECLARE_FINAL_TYPE (AdwViewStackPages, adw_view_stack_pages, ADW, VIEW_STACK_PAGES, GObject)
@@ -473,8 +567,8 @@ adw_view_stack_pages_new (AdwViewStack *stack)
   return pages;
 }
 
-static AdwViewStackPage *
-find_page_for_widget (AdwViewStack *self,
+static GList *
+find_link_for_widget (AdwViewStack *self,
                       GtkWidget    *child)
 {
   AdwViewStackPage *page;
@@ -484,12 +578,24 @@ find_page_for_widget (AdwViewStack *self,
     page = l->data;
 
     if (page->widget == child)
-      return page;
+      return l;
   }
 
   return NULL;
 }
 
+static AdwViewStackPage *
+find_page_for_widget (AdwViewStack *self,
+                      GtkWidget    *child)
+{
+  GList *l = find_link_for_widget (self, child);
+
+  if (l)
+    return l->data;
+
+  return NULL;
+}
+
 static AdwViewStackPage *
 find_page_for_name (AdwViewStack *self,
                     const char   *name)
@@ -625,6 +731,10 @@ update_child_visible (AdwViewStack     *self,
     set_visible_child (self, page);
   else if (self->visible_child == page && !visible)
     set_visible_child (self, NULL);
+
+  gtk_accessible_update_state (GTK_ACCESSIBLE (page),
+                               GTK_ACCESSIBLE_STATE_HIDDEN, !visible,
+                               -1);
 }
 
 static void
@@ -660,6 +770,14 @@ add_page (AdwViewStack     *self,
     }
   }
 
+  if (self->children) {
+    AdwViewStackPage *prev_last = g_list_last (self->children)->data;
+
+    prev_last->next_page = page;
+  } else {
+    page->next_page = NULL;
+  }
+
   self->children = g_list_append (self->children, g_object_ref (page));
 
   gtk_widget_set_child_visible (page->widget, FALSE);
@@ -712,11 +830,14 @@ stack_remove (AdwViewStack  *self,
 {
   AdwViewStackPage *page;
   gboolean was_visible;
+  GList *l;
 
-  page = find_page_for_widget (self, child);
-  if (!page)
+  l = find_link_for_widget (self, child);
+  if (!l)
     return;
 
+  page = l->data;
+
   g_signal_handlers_disconnect_by_func (child,
                                         stack_child_visibility_notify_cb,
                                         self);
@@ -730,8 +851,16 @@ stack_remove (AdwViewStack  *self,
 
   g_clear_object (&page->widget);
 
+  l = l->prev;
+
   self->children = g_list_remove (self->children, page);
 
+  if (l) {
+    AdwViewStackPage *prev_page = l->data;
+
+    prev_page->next_page = page->next_page;
+  }
+
   g_object_unref (page);
 
   if (!in_dispose &&
@@ -898,12 +1027,13 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
   /**
    * AdwViewStack:hhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_hhomogeneous org.gtk.Property.set=adw_view_stack_set_hhomogeneous)
    *
-   * Whether the stack allocates the same width for all children.
+   * Whether the stack is horizontally homogeneous.
+   *
+   * If the stack is horizontally homogeneous, it allocates the same width for
+   * all children.
    *
    * If it's `FALSE`, the stack may change width when a different child becomes
    * visible.
-   *
-   * Since: 1.0
    */
   props[PROP_HHOMOGENEOUS] =
     g_param_spec_boolean ("hhomogeneous", NULL, NULL,
@@ -913,12 +1043,13 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
   /**
    * AdwViewStack:vhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_vhomogeneous org.gtk.Property.set=adw_view_stack_set_vhomogeneous)
    *
-   * Whether the stack allocates the same height for all children.
+   * Whether the stack is vertically homogeneous.
+   *
+   * If the stack is vertically homogeneous, it allocates the same height for
+   * all children.
    *
    * If it's `FALSE`, the stack may change height when a different child becomes
    * visible.
-   *
-   * Since: 1.0
    */
   props[PROP_VHOMOGENEOUS] =
     g_param_spec_boolean ("vhomogeneous", NULL, NULL,
@@ -929,8 +1060,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    * AdwViewStack:visible-child: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child org.gtk.Property.set=adw_view_stack_set_visible_child)
    *
    * The widget currently visible in the stack.
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_CHILD] =
     g_param_spec_object ("visible-child", NULL, NULL,
@@ -943,8 +1072,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    * The name of the widget currently visible in the stack.
    *
    * See [property@ViewStack:visible-child].
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_CHILD_NAME] =
     g_param_spec_string ("visible-child-name", NULL, NULL,
@@ -1002,6 +1129,23 @@ adw_view_stack_buildable_init (GtkBuildableIface *iface)
   iface->add_child = adw_view_stack_buildable_add_child;
 }
 
+static GtkAccessible *
+adw_view_stack_accessible_get_first_accessible_child (GtkAccessible *accessible)
+{
+  AdwViewStack *self = ADW_VIEW_STACK (accessible);
+
+  if (self->children)
+    return self->children->data;
+
+  return NULL;
+}
+
+static void
+adw_view_stack_accessible_init (GtkAccessibleInterface *iface)
+{
+  iface->get_first_accessible_child = adw_view_stack_accessible_get_first_accessible_child;
+}
+
 /**
  * adw_view_stack_page_get_child: (attributes org.gtk.Method.get_property=child)
  * @self: a view stack page
@@ -1009,8 +1153,6 @@ adw_view_stack_buildable_init (GtkBuildableIface *iface)
  * Gets the stack child to which @self belongs.
  *
  * Returns: (transfer none): the child to which @self belongs
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_page_get_child (AdwViewStackPage *self)
@@ -1020,178 +1162,6 @@ adw_view_stack_page_get_child (AdwViewStackPage *self)
   return self->widget;
 }
 
-/**
- * adw_view_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
- * @self: a view stack page
- *
- * Gets whether @self is visible in its `AdwViewStack`.
- *
- * This is independent from the [property@Gtk.Widget:visible]
- * property of its widget.
- *
- * Returns: whether @self is visible
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_page_get_visible (AdwViewStackPage *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
-
-  return self->visible;
-}
-
-/**
- * adw_view_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
- * @self: a view stack page
- * @visible: whether @self is visible
- *
- * Sets whether @page is visible in its `AdwViewStack`.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_visible (AdwViewStackPage *self,
-                                 gboolean          visible)
-{
-  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
-
-  visible = !!visible;
-
-  if (visible == self->visible)
-    return;
-
-  self->visible = visible;
-
-  if (self->widget && gtk_widget_get_parent (self->widget))
-    update_child_visible (ADW_VIEW_STACK (gtk_widget_get_parent (self->widget)), self);
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_VISIBLE]);
-}
-
-/**
- * adw_view_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
- * @self: a view stack page
- *
- * Gets whether the page is marked as “needs attention”.
- *
- * Returns: whether the page needs attention
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
-
-  return self->needs_attention;
-}
-
-/**
- * adw_view_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
- * @self: a view stack page
- * @needs_attention: the new value to set
- *
- * Sets whether the page is marked as “needs attention”.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
-                                         gboolean          needs_attention)
-{
-  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
-
-  needs_attention = !!needs_attention;
-
-  if (needs_attention == self->needs_attention)
-    return;
-
-  self->needs_attention = needs_attention;
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_NEEDS_ATTENTION]);
-}
-
-/**
- * adw_view_stack_page_get_badge_number: (attributes org.gtk.Method.get_property=badge-number)
- * @self: a view stack page
- *
- * Gets the badge number for this page.
- *
- * Returns: the badge number for this page
- *
- * Since: 1.0
- */
-guint
-adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), 0);
-
-  return self->badge_number;
-}
-
-/**
- * adw_view_stack_page_set_badge_number: (attributes org.gtk.Method.set_property=badge-number)
- * @self: a view stack page
- * @badge_number: the new value to set
- *
- * Sets the badge number for this page.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
-                                      guint             badge_number)
-{
-  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
-
-  if (badge_number == self->badge_number)
-    return;
-
-  self->badge_number = badge_number;
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_BADGE_NUMBER]);
-}
-
-/**
- * adw_view_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
- * @self: a view stack page
- *
- * Gets whether underlines in the page title indicate mnemonics.
- *
- * Returns: whether underlines in the page title indicate mnemonics
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
-{
-  return self->use_underline;
-}
-
-/**
- * adw_view_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
- * @self: a view stack page
- * @use_underline: the new value to set
- *
- * Sets whether underlines in the page title indicate mnemonics.
- *
- * Since: 1.0
- */
-void
-adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
-                                       gboolean          use_underline)
-{
-  use_underline = !!use_underline;
-
-  if (use_underline == self->use_underline)
-    return;
-
-  self->use_underline = use_underline;
-
-  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_USE_UNDERLINE]);
-}
-
 /**
  * adw_view_stack_page_get_name: (attributes org.gtk.Method.get_property=name)
  * @self: a view stack page
@@ -1199,8 +1169,6 @@ adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
  * Gets the name of the page.
  *
  * Returns: (nullable): the name of the page
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_name (AdwViewStackPage *self)
@@ -1216,8 +1184,6 @@ adw_view_stack_page_get_name (AdwViewStackPage *self)
  * @name: (nullable): the page name
  *
  * Sets the name of the page.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_name (AdwViewStackPage *self,
@@ -1267,8 +1233,6 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
  * Gets the page title.
  *
  * Returns: (nullable): the page title
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_title (AdwViewStackPage *self)
@@ -1284,8 +1248,6 @@ adw_view_stack_page_get_title (AdwViewStackPage *self)
  * @title: (nullable): the page title
  *
  * Sets the page title.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_title (AdwViewStackPage *self,
@@ -1300,6 +1262,45 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
   self->title = g_strdup (title);
 
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_TITLE]);
+
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL, self->title,
+                                  -1);
+}
+
+/**
+ * adw_view_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * @self: a view stack page
+ *
+ * Gets whether underlines in the page title indicate mnemonics.
+ *
+ * Returns: whether underlines in the page title indicate mnemonics
+ */
+gboolean
+adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
+{
+  return self->use_underline;
+}
+
+/**
+ * adw_view_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * @self: a view stack page
+ * @use_underline: the new value to set
+ *
+ * Sets whether underlines in the page title indicate mnemonics.
+ */
+void
+adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
+                                       gboolean          use_underline)
+{
+  use_underline = !!use_underline;
+
+  if (use_underline == self->use_underline)
+    return;
+
+  self->use_underline = use_underline;
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_USE_UNDERLINE]);
 }
 
 /**
@@ -1309,8 +1310,6 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
  * Gets the icon name of the page.
  *
  * Returns: (nullable): the icon name of the page
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
@@ -1326,8 +1325,6 @@ adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
  * @icon_name: (nullable): the icon name
  *
  * Sets the icon name of the page.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
@@ -1344,14 +1341,143 @@ adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_ICON_NAME]);
 }
 
+/**
+ * adw_view_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
+ * @self: a view stack page
+ *
+ * Gets whether the page requires the user attention.
+ *
+ * Returns: whether the page needs attention
+ */
+gboolean
+adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
+
+  return self->needs_attention;
+}
+
+/**
+ * adw_view_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
+ * @self: a view stack page
+ * @needs_attention: the new value to set
+ *
+ * Sets whether the page requires the user attention.
+ *
+ * [class@ViewSwitcher] will display it as a dot next to the page icon.
+ */
+void
+adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
+                                         gboolean          needs_attention)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
+
+  needs_attention = !!needs_attention;
+
+  if (needs_attention == self->needs_attention)
+    return;
+
+  self->needs_attention = needs_attention;
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_NEEDS_ATTENTION]);
+}
+
+/**
+ * adw_view_stack_page_get_badge_number: (attributes org.gtk.Method.get_property=badge-number)
+ * @self: a view stack page
+ *
+ * Gets the badge number for this page.
+ *
+ * Returns: the badge number for this page
+ */
+guint
+adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), 0);
+
+  return self->badge_number;
+}
+
+/**
+ * adw_view_stack_page_set_badge_number: (attributes org.gtk.Method.set_property=badge-number)
+ * @self: a view stack page
+ * @badge_number: the new value to set
+ *
+ * Sets the badge number for this page.
+ *
+ * [class@ViewSwitcher] can display it as a badge next to the page icon. It is
+ * commonly used to display a number of unread items within the page.
+ *
+ * It can be used together with [property@ViewStack{age}:needs-attention].
+ */
+void
+adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
+                                      guint             badge_number)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
+
+  if (badge_number == self->badge_number)
+    return;
+
+  self->badge_number = badge_number;
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_BADGE_NUMBER]);
+}
+
+/**
+ * adw_view_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
+ * @self: a view stack page
+ *
+ * Gets whether @self is visible in its `AdwViewStack`.
+ *
+ * This is independent from the [property@Gtk.Widget:visible]
+ * property of its widget.
+ *
+ * Returns: whether @self is visible
+ */
+gboolean
+adw_view_stack_page_get_visible (AdwViewStackPage *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGE (self), FALSE);
+
+  return self->visible;
+}
+
+/**
+ * adw_view_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
+ * @self: a view stack page
+ * @visible: whether @self is visible
+ *
+ * Sets whether @page is visible in its `AdwViewStack`.
+ *
+ * This is independent from the [property@Gtk.Widget:visible] property of
+ * [property@ViewStackPage:child].
+ */
+void
+adw_view_stack_page_set_visible (AdwViewStackPage *self,
+                                 gboolean          visible)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
+
+  visible = !!visible;
+
+  if (visible == self->visible)
+    return;
+
+  self->visible = visible;
+
+  if (self->widget && gtk_widget_get_parent (self->widget))
+    update_child_visible (ADW_VIEW_STACK (gtk_widget_get_parent (self->widget)), self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_VISIBLE]);
+}
+
 /**
  * adw_view_stack_new:
  *
  * Creates a new `AdwViewStack`.
  *
  * Returns: the newly created `AdwViewStack`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_new (void)
@@ -1367,8 +1493,6 @@ adw_view_stack_new (void)
  * Adds a child to @self.
  *
  * Returns: (transfer none): the [class@ViewStackPage] for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add (AdwViewStack   *self,
@@ -1391,8 +1515,6 @@ adw_view_stack_add (AdwViewStack   *self,
  * The child is identified by the @name.
  *
  * Returns: (transfer none): the `AdwViewStackPage` for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add_named (AdwViewStack   *self,
@@ -1418,8 +1540,6 @@ adw_view_stack_add_named (AdwViewStack   *self,
  * [class@ViewSwitcher] to represent @child, so it should be short.
  *
  * Returns: (transfer none): the `AdwViewStackPage` for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add_titled (AdwViewStack   *self,
@@ -1469,8 +1589,6 @@ adw_view_stack_add_titled_with_icon (AdwViewStack *self,
  * @child: the child to remove
  *
  * Removes a child widget from @self.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_remove (AdwViewStack  *self,
@@ -1504,8 +1622,6 @@ adw_view_stack_remove (AdwViewStack  *self,
  * Gets the [class@ViewStackPage] object for @child.
  *
  * Returns: (transfer none): the page object for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_get_page (AdwViewStack  *self,
@@ -1525,8 +1641,6 @@ adw_view_stack_get_page (AdwViewStack  *self,
  * Finds the child with @name in @self.
  *
  * Returns: (transfer none) (nullable): the requested child
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_get_child_by_name (AdwViewStack *self,
@@ -1549,8 +1663,6 @@ adw_view_stack_get_child_by_name (AdwViewStack *self,
  * Gets the currently visible child of @self, .
  *
  * Returns: (transfer none) (nullable): the visible child
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_get_visible_child (AdwViewStack *self)
@@ -1566,8 +1678,6 @@ adw_view_stack_get_visible_child (AdwViewStack *self)
  * @child: a child of @self
  *
  * Makes @child the visible child of @self.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_set_visible_child (AdwViewStack *self,
@@ -1597,8 +1707,6 @@ adw_view_stack_set_visible_child (AdwViewStack *self,
  * Returns the name of the currently visible child of @self.
  *
  * Returns: (transfer none) (nullable): the name of the visible child
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_get_visible_child_name (AdwViewStack *self)
@@ -1615,7 +1723,7 @@ adw_view_stack_get_visible_child_name (AdwViewStack *self)
  *
  * Makes the child with @name visible.
  *
- * Since: 1.0
+ * See [property@ViewStack:visible-child].
  */
 void
 adw_view_stack_set_visible_child_name (AdwViewStack *self,
@@ -1640,6 +1748,22 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
     set_visible_child (self, page);
 }
 
+/**
+ * adw_view_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * @self: a view stack
+ *
+ * Gets whether @self is horizontally homogeneous.
+ *
+ * Returns: whether @self is horizontally homogeneous
+ */
+gboolean
+adw_view_stack_get_hhomogeneous (AdwViewStack *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
+
+  return self->homogeneous[GTK_ORIENTATION_HORIZONTAL];
+}
+
 /**
  * adw_view_stack_set_hhomogeneous: (attributes org.gtk.Method.set_property=hhomogeneous)
  * @self: a view stack
@@ -1647,7 +1771,11 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
  *
  * Sets @self to be horizontally homogeneous or not.
  *
- * Since: 1.0
+ * If the stack is horizontally homogeneous, it allocates the same width for
+ * all children.
+ *
+ * If it's `FALSE`, the stack may change width when a different child becomes
+ * visible.
  */
 void
 adw_view_stack_set_hhomogeneous (AdwViewStack *self,
@@ -1669,21 +1797,19 @@ adw_view_stack_set_hhomogeneous (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * adw_view_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
  * @self: a view stack
  *
- * Gets whether @self is horizontally homogeneous.
+ * Gets whether @self is vertically homogeneous.
  *
- * Returns: whether @self is horizontally homogeneous
- *
- * Since: 1.0
+ * Returns: whether @self is vertically homogeneous
  */
 gboolean
-adw_view_stack_get_hhomogeneous (AdwViewStack *self)
+adw_view_stack_get_vhomogeneous (AdwViewStack *self)
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
 
-  return self->homogeneous[GTK_ORIENTATION_HORIZONTAL];
+  return self->homogeneous[GTK_ORIENTATION_VERTICAL];
 }
 
 /**
@@ -1693,7 +1819,11 @@ adw_view_stack_get_hhomogeneous (AdwViewStack *self)
  *
  * Sets @self to be vertically homogeneous or not.
  *
- * Since: 1.0
+ * If the stack is vertically homogeneous, it allocates the same height for
+ * all children.
+ *
+ * If it's `FALSE`, the stack may change height when a different child becomes
+ * visible.
  */
 void
 adw_view_stack_set_vhomogeneous (AdwViewStack *self,
@@ -1714,24 +1844,6 @@ adw_view_stack_set_vhomogeneous (AdwViewStack *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VHOMOGENEOUS]);
 }
 
-/**
- * adw_view_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
- * @self: a view stack
- *
- * Gets whether @self is vertically homogeneous.
- *
- * Returns: whether @self is vertically homogeneous
- *
- * Since: 1.0
- */
-gboolean
-adw_view_stack_get_vhomogeneous (AdwViewStack *self)
-{
-  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
-
-  return self->homogeneous[GTK_ORIENTATION_VERTICAL];
-}
-
 /**
  * adw_view_stack_get_pages: (attributes org.gtk.Method.get_property=pages)
  * @self: a view stack
@@ -1743,8 +1855,6 @@ adw_view_stack_get_vhomogeneous (AdwViewStack *self)
  * page.
  *
  * Returns: (transfer full): a `GtkSelectionModel` for the stack's children
- *
- * Since: 1.0
  */
 GtkSelectionModel *
 adw_view_stack_get_pages (AdwViewStack *self)
