@@ -7,8 +7,8 @@
 
 #include "config.h"
 
-#include "adw-macros-private.h"
 #include "adw-view-switcher-title.h"
+
 #include "adw-squeezer.h"
 #include "adw-window-title.h"
 
@@ -50,10 +50,10 @@
  *         <object class="AdwViewSwitcherTitle" id="title">
  *           <property name="stack">stack</property>
  *         </object>
- *       </child>
+ *       </property>
  *     </object>
  *   </property>
- *   <property>
+ *   <property name="child">
  *     <object class="GtkBox">
  *       <property name="orientation">vertical</property>
  *       <child>
@@ -75,8 +75,6 @@
  * ## CSS nodes
  *
  * `AdwViewSwitcherTitle` has a single CSS node with name `viewswitchertitle`.
- *
- * Since: 1.0
  */
 
 enum {
@@ -284,8 +282,6 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * AdwViewSwitcherTitle:stack: (attributes org.gtk.Property.get=adw_view_switcher_title_get_stack org.gtk.Property.set=adw_view_switcher_title_set_stack)
    *
    * The stack the view switcher controls.
-   *
-   * Since: 1.0
    */
   props[PROP_STACK] =
     g_param_spec_object ("stack", NULL, NULL,
@@ -297,10 +293,8 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    *
    * The title to display.
    *
-   * The title should give a user additional details. A good title should not
-   * include the application name.
-   *
-   * Since: 1.0
+   * The title typically identifies the current view or content item, and
+   * generally does not use the application name.
    */
   props[PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
@@ -312,9 +306,7 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    *
    * The subtitle to display.
    *
-   * The subtitle should give a user additional details.
-   *
-   * Since: 1.0
+   * The subtitle should give the user additional details.
    */
   props[PROP_SUBTITLE] =
     g_param_spec_string ("subtitle", NULL, NULL,
@@ -332,8 +324,6 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    *
    * This can be used e.g. to ensure the view switcher is hidden below a certain
    * window width, or any other constraint you find suitable.
-   *
-   * Since: 1.0
    */
   props[PROP_VIEW_SWITCHER_ENABLED] =
     g_param_spec_boolean ("view-switcher-enabled", NULL, NULL,
@@ -347,8 +337,6 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    *
    * If the title is visible, it means the view switcher is hidden an it may be
    * wanted to show an alternative switcher, e.g. a [class@ViewSwitcherBar].
-   *
-   * Since: 1.0
    */
   props[PROP_TITLE_VISIBLE] =
     g_param_spec_boolean ("title-visible", NULL, NULL,
@@ -388,8 +376,6 @@ adw_view_switcher_title_init (AdwViewSwitcherTitle *self)
  * Creates a new `AdwViewSwitcherTitle`.
  *
  * Returns: the newly created `AdwViewSwitcherTitle`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_switcher_title_new (void)
@@ -404,8 +390,6 @@ adw_view_switcher_title_new (void)
  * Gets the stack controlled by @self.
  *
  * Returns: (nullable) (transfer none): the stack
- *
- * Since: 1.0
  */
 AdwViewStack *
 adw_view_switcher_title_get_stack (AdwViewSwitcherTitle *self)
@@ -421,8 +405,6 @@ adw_view_switcher_title_get_stack (AdwViewSwitcherTitle *self)
  * @stack: (nullable): a stack
  *
  * Sets the stack controlled by @self.
- *
- * Since: 1.0
  */
 void
 adw_view_switcher_title_set_stack (AdwViewSwitcherTitle *self,
@@ -464,8 +446,6 @@ adw_view_switcher_title_set_stack (AdwViewSwitcherTitle *self,
  * Gets the title of @self.
  *
  * Returns: the title
- *
- * Since: 1.0
  */
 const char *
 adw_view_switcher_title_get_title (AdwViewSwitcherTitle *self)
@@ -482,7 +462,8 @@ adw_view_switcher_title_get_title (AdwViewSwitcherTitle *self)
  *
  * Sets the title of @self.
  *
- * Since: 1.0
+ * The title typically identifies the current view or content item, and
+ * generally does not use the application name.
  */
 void
 adw_view_switcher_title_set_title (AdwViewSwitcherTitle *self,
@@ -505,8 +486,6 @@ adw_view_switcher_title_set_title (AdwViewSwitcherTitle *self,
  * Gets the subtitle of @self.
  *
  * Returns: the subtitle
- *
- * Since: 1.0
  */
 const char *
 adw_view_switcher_title_get_subtitle (AdwViewSwitcherTitle *self)
@@ -523,7 +502,7 @@ adw_view_switcher_title_get_subtitle (AdwViewSwitcherTitle *self)
  *
  * Sets the subtitle of @self.
  *
- * Since: 1.0
+ * The subtitle should give the user additional details.
  */
 void
 adw_view_switcher_title_set_subtitle (AdwViewSwitcherTitle *self,
@@ -546,8 +525,6 @@ adw_view_switcher_title_set_subtitle (AdwViewSwitcherTitle *self,
  * Gets whether @self's view switcher is enabled.
  *
  * Returns: whether the view switcher is enabled
- *
- * Since: 1.0
  */
 gboolean
 adw_view_switcher_title_get_view_switcher_enabled (AdwViewSwitcherTitle *self)
@@ -564,7 +541,12 @@ adw_view_switcher_title_get_view_switcher_enabled (AdwViewSwitcherTitle *self)
  *
  * Sets whether @self's view switcher is enabled.
  *
- * Since: 1.0
+ * If it is disabled, the title will be displayed instead. This allows to
+ * programmatically hide the view switcher even if it fits in the available
+ * space.
+ *
+ * This can be used e.g. to ensure the view switcher is hidden below a certain
+ * window width, or any other constraint you find suitable.
  */
 void
 adw_view_switcher_title_set_view_switcher_enabled (AdwViewSwitcherTitle *self,
@@ -589,9 +571,10 @@ adw_view_switcher_title_set_view_switcher_enabled (AdwViewSwitcherTitle *self,
  *
  * Gets whether the title of @self is currently visible.
  *
- * Returns: whether the title of @self is currently visible
+ * If the title is visible, it means the view switcher is hidden an it may be
+ * wanted to show an alternative switcher, e.g. a [class@ViewSwitcherBar].
  *
- * Since: 1.0
+ * Returns: whether the title of @self is currently visible
  */
 gboolean
 adw_view_switcher_title_get_title_visible (AdwViewSwitcherTitle *self)
